# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.common.utils import extract_url_parts
from otcextensions.sdk.function_graph.v2 import function as _function
from otcextensions.sdk.function_graph.v2 import function_invocation as _fi
from otcextensions.sdk.function_graph.v2 import quota as _quota
from otcextensions.sdk.function_graph.v2 import dependency as _d
from otcextensions.sdk.function_graph.v2 import event as _event
from otcextensions.sdk.function_graph.v2 import alias as _alias
from otcextensions.sdk.function_graph.v2 import version as _version
from otcextensions.sdk.function_graph.v2 import metric as _metric
from otcextensions.sdk.function_graph.v2 import log as _log
from otcextensions.sdk.function_graph.v2 import template as _t
from otcextensions.sdk.function_graph.v2 import reserved_instance as _r
from otcextensions.sdk.function_graph.v2 import export_function as _export
from otcextensions.sdk.function_graph.v2 import import_function as _import
from otcextensions.sdk.function_graph.v2 import trigger as _trigger
from otcextensions.sdk.function_graph.v2 import async_notification as _async


class Proxy(proxy.Proxy):
    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return extract_url_parts(url, project_id)

    # ======== Function Methods ========

    def create_function(self, **attrs):
        """Create a new function from attributes.

        :param dict attrs: Keyword arguments to create a Function.
        :returns: The created Function instance.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.function.Function`
        """
        return self._create(_function.Function, **attrs)

    def delete_function(self, function, ignore_missing=True):
        """Delete a function.

        :param ignore_missing:
        :param function: The instance of the Function to delete.
        :returns: ``None``
        """
        function = self._get_resource(_function.Function, function)
        return function._delete_function(self, function)

    def functions(self, **query):
        """List all functions with optional filters.

        :param dict query: Query parameters to filter the function list.
        :returns: A generator of Function instances.
        """
        return self._list(_function.Function, **query)

    def get_function_code(self, function):
        """Get details of a function code.

        :param function: The URN or instance of the Function to retrieve.
        :returns: The Function instance.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.function.Function`
        """
        function = self._get_resource(_function.Function, function)
        return function._get_function_code(self, function)

    def get_function_metadata(self, function):
        """Get details of a function metadata.

        :param function: The URN or instance of the Function to retrieve.
        :returns: The Function instance.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.function.Function`
        """
        function = self._get_resource(_function.Function, function)
        return function._get_function_metadata(self, function)

    def get_resource_tags(self, function):
        """Get a function resource tags.

        :param function: The URN or instance of the Function to retrieve.
        :returns: The Function instance.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.function.Function`
        """
        function = self._get_resource(_function.Function, function)
        return function._get_resource_tags(self, function)

    def create_resource_tags(self, function, tags):
        """Create function resource tags.

        :param tags: list of tags
        :param function: The URN or instance of the Function to create tags.
        :returns: ``None``
        """
        function = self._get_resource(_function.Function, function)
        return function._create_resource_tags(self, function, tags)

    def delete_resource_tags(self, function, tags):
        """Delete function resource tags.

        :param tags: list of tags
        :param function: The URN or instance of the Function
               from which need to delete tags.
        :returns: ``None``
        """
        function = self._get_resource(_function.Function, function)
        return function._delete_resource_tags(self, function, tags)

    def update_pin_status(self, function):
        """Update a function's pin status.

        :param function: The URN or instance of the Function to update.
        :returns: The updated Function pin status.
        :returns: ``None``
        """
        function = self._get_resource(_function.Function, function)
        return function._update_pin_status(self, function)

    def update_function_code(self, function, **attrs):
        """Update a function code.

        :param function: The URN or instance of the Function to update.
        :param attrs: Attributes for updating the function code. These include:
               code_type: Function code type. Options:
               * `inline`: Inline code.
               * `zip`: ZIP file.
               * `obs`: Function code stored in an OBS bucket.
               * `jar`: JAR file (mainly for Java functions).

               code_url: If `code_type` is set to `obs`, enter the OBS URL
               of the function code package. Leave this parameter blank if
               `code_type` is not `obs`.

               code_filename: Name of the function file. This parameter
               is mandatory only when `code_type` is set to `jar` or `zip`.

               func_code: Response body of the `FuncCode` struct.

               depend_version_list: Dependency version IDs.

        :rtype: :class:`~otcextensions.sdk.function_graph.v2.function.Function`
        """
        function = self._get_resource(_function.Function, function)
        return function._update_function_code(self, function, **attrs)

    def update_function_metadata(self, function, **attrs):
        """Update a function metadata.

        :param function: The URN or instance of the Function to update.
        :returns: The updated Function pin status.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.function.Function`
        """
        function = self._get_resource(_function.Function, function)
        return function._update_function_metadata(self, function, **attrs)

    def update_max_instances(self, function, instances):
        """Update a function max instances number.

        :param instances: Maximum number of instances.
        :param function: The URN or instance of the Function to update.
        :returns: The updated Function pin status.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.function.Function`
        """
        function = self._get_resource(_function.Function, function)
        return function._update_max_instances(self, function, instances)

    # ======== Function Invocations Methods ========

    def executing_function_synchronously(self, func_urn, **attrs):
        """Execute a function synchronously.

        :param func_urn: The URN of the Function to run
        :param attrs: The request parameter as a key pair ("k":"v")
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.
            function_invocation.FunctionInvocation`
        """
        fi = self._get_resource(
            _fi.FunctionInvocation, func_urn,
            func_urn=func_urn.rpartition(":")[0]
        )
        return fi._invocation(self, 'invocations', **attrs)

    def executing_function_asynchronously(self, func_urn, **attrs):
        """Execute a function asynchronously.

        :param func_urn: The URN of the Function to run
        :param attrs: The request parameter as a key pair ("k":"v")
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.
            function_invocation.FunctionInvocation`
        """
        fi = self._get_resource(
            _fi.FunctionInvocation, func_urn,
            func_urn=func_urn.rpartition(":")[0]
        )
        return fi._invocation(self, 'invocations-async', **attrs)

    # ======== Function Quotas Methods ========

    def quotas(self):
        """List all quotas.

        :returns: A generator of Quota instances.
        """
        return self._list(_quota.Quota)

    # ======== Function Dependencies Methods ========

    def dependencies(self, **query):
        """List all dependencies.

        :param dict query: Query parameters to filter the dependencies list.
        :returns: A generator of Dependency instances.
        """
        return self._list(_d.Dependency, **query)

    def create_dependency_version(self, **attrs):
        """Create a new dependency from attributes.

        :param dict attrs: Keyword arguments to create a Function.
        :returns: The created Dependency instance.
        :rtype:
            :class:`~otcextensions.sdk.function_graph.v2.dependency.Dependency`
        """
        base_path = "/fgs/dependencies/version"
        return self._create(_d.Dependency, **attrs, base_path=base_path)

    def delete_dependency_version(self, dependency, ignore_missing=True):
        """Delete a dependency.

        :param ignore_missing:
        :param dependency: The instance of the Dependency to delete.
        :returns: ``None``
        """
        dep = self._get_resource(_d.Dependency, dependency)
        return dep._delete_version(self, dependency)

    def dependency_versions(self, dependency, **query):
        """List all dependency versions.

        :param dependency: Dependency instance
        :param dict query: Query parameters to filter the versions list.
        :returns: A generator of Dependency instances.
        """
        base_path = f"/fgs/dependencies/{dependency.dep_id}/version"
        return self._list(_d.Dependency, **query, base_path=base_path)

    def get_dependency_version(self, dependency):
        """List all dependency versions.

        :param dependency: Dependency instance
        :returns: A generator of Dependency instances.
        """
        base_path = (f"/fgs/dependencies/{dependency.dep_id}"
                     f"/version/{dependency.version}")
        return self._get(_d.Dependency, base_path=base_path, requires_id=False)

    # ======== Test Events Methods ========

    def create_event(self, function, **attrs):
        """Create a new event from attributes.

        :param function: The URN or instance of the Function to create
            event in.
        :param dict attrs: Keyword arguments to create an Event.
        :returns: The created Event instance.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.event.Event`
        """
        function = self._get_resource(_function.Function, function)
        function_urn = function.func_urn.rpartition(":")[0]
        return self._create(
            _event.Event, function_urn=function_urn, **attrs
        )

    def delete_event(self, function, event, ignore_missing=True):
        """Delete an event.

        :param function: The URN or instance of the Function to delete
            event from.
        :param ignore_missing: When False,
            `openstack.exceptions.ResourceNotFound`
            will be raised when the tag does not exist.
            When True, no exception will be set when attempting
            to delete a nonexistent event.
        :param event: The instance of the Event to delete.
        :returns: ``None``
        """
        function = self._get_resource(_function.Function, function)
        function_urn = function.func_urn.rpartition(":")[0]
        return self._delete(
            _event.Event, event,
            function_urn=function_urn, ignore_missing=ignore_missing
        )

    def events(self, func_urn):
        """List all events.

        :param func_urn: The URN of the Function to fetch
            events from.
        :returns: A generator of Event instances.
        """
        function_urn = func_urn.rpartition(":")[0]
        return self._list(_event.Event, function_urn=function_urn)

    def get_event(self, function, event):
        """Get one event by ID.

        :param event: key id or an instance of
            :class:`~otcextensions.sdk.function_graph.v2.event.Event`
        :param function: The value can be the ID of a function or
            a :class:`~otcextensions.sdk.function_graph.v2.function.Function`
            instance.
        :returns: instance of
            :class:`~otcextensions.sdk.function_graph.v2.event.Event`
        """
        function = self._get_resource(_function.Function, function)
        return self._get(
            _event.Event, event,
            function_urn=function.func_urn
        )

    def update_event(self, function, event, **attrs):
        """Update an event from attributes.

        :param event: key id or an instance of
            :class:`~otcextensions.sdk.function_graph.v2.event.Event`
        :param function: The URN or instance of the Function to update
            event in.
        :param dict attrs: Keyword arguments to update an Event.
        :returns: The updated Event instance.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.event.Event`
        """
        function = self._get_resource(_function.Function, function)
        function_urn = function.func_urn.rpartition(":")[0]
        return self._update(
            _event.Event, event, function_urn=function_urn, **attrs
        )

    # ======== Versions Methods ========
    def versions(self, func_urn):
        """List all published versions.

        :param func_urn: The URN of the Function to fetch
            events from.
        :returns: A generator of Version instances.
        """
        function_urn = func_urn.rpartition(":")[0]
        return self._list(_version.Version, function_urn=function_urn)

    def publish_version(self, function, **attrs):
        """Publish a new version from attributes.

        :param function: The URN or instance of the Function to create
            event in.
        :param dict attrs: Keyword arguments to publish a Version.
        :returns: The created Version instance.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.version.Version`
        """
        function = self._get_resource(_function.Function, function)
        function_urn = function.func_urn.rpartition(":")[0]
        return self._create(
            _version.Version, function_urn=function_urn, **attrs
        )

    # ======== Aliases Methods ========

    def aliases(self, func_urn):
        """List all aliases.

        :param func_urn: The URN of the Function to fetch
            events from.
        :returns: A generator of Alias instances.
        """
        function_urn = func_urn.rpartition(":")[0]
        return self._list(_alias.Alias, function_urn=function_urn)

    def create_alias(self, function, **attrs):
        """Create a new alias from attributes.

        :param function: The URN or instance of the Function to create
            event in.
        :param dict attrs: Keyword arguments to create an Alias.
        :returns: The created Event instance.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.alias.Alias`
        """
        function = self._get_resource(_function.Function, function)
        function_urn = function.id.rpartition(":")[0]
        return self._create(
            _alias.Alias, function_urn=function_urn, **attrs
        )

    def delete_alias(self, function, alias, ignore_missing=True):
        """Delete an alias.

        :param function: The URN or instance of the Function to delete
            alias from.
        :param ignore_missing: When False,
            `openstack.exceptions.ResourceNotFound`
            will be raised when the tag does not exist.
            When True, no exception will be set when attempting
            to delete a nonexistent event.
        :param alias: The instance of the Alias to delete.
        :returns: ``None``
        """
        function = self._get_resource(_function.Function, function)
        function_urn = function.func_urn.rpartition(":")[0]
        return self._delete(
            _alias.Alias, alias,
            function_urn=function_urn, ignore_missing=ignore_missing
        )

    def update_alias(self, function, alias, **attrs):
        """Update an alias from attributes.

        :param alias: key id or an instance of
            :class:`~otcextensions.sdk.function_graph.v2.alias.Alias`
        :param function: The URN or instance of the Function to update
            alias in.
        :param dict attrs: Keyword arguments to update an Alias.
        :returns: The updated Alias instance.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.alias.Alias`
        """
        function = self._get_resource(_function.Function, function)
        a = self._get_resource(_alias.Alias, alias)
        return a._update_alias(self, function, alias, **attrs)

    def get_alias(self, function, alias):
        """Get one alias by ID.

        :param alias: key id or an instance of
            :class:`~otcextensions.sdk.function_graph.v2.alias.Alias`
        :param function: The value can be the ID of a function or
            a :class:`~otcextensions.sdk.function_graph.v2.function.Function`
            instance.
        :returns: instance of
            :class:`~otcextensions.sdk.function_graph.v2.alias.Alias`
        """
        function = self._get_resource(_function.Function, function)
        return self._get(
            _alias.Alias, alias,
            function_urn=function.func_urn
        )

    # ======== Metric Methods ========

    def metrics(self, **query):
        """List all tenant-level function statistics.

        :returns: A generator of Metric instances.
        """
        return self._list(_metric.Metric, **query)

    def function_metrics(self, function, period):
        """List all metrics of a function in a specified period.

        :returns: A generator of Metric instances.
        """
        function = self._get_resource(_function.Function, function)
        func_urn = function.func_urn.rpartition(":")[0]
        base_path = f'/fgs/functions/{func_urn}/statistics/{period}'
        return self._list(_metric.Metric, base_path=base_path)

    # ======== Log Methods ========

    def get_lts_log_settings(self, function):
        """Get log group and stream settings of a function.

        :param function: The value can be the ID of a function or
            a :class:`~otcextensions.sdk.function_graph.v2.function.Function`
            instance.
        :returns: instance of
            :class:`~otcextensions.sdk.function_graph.v2.log.Log`
        """
        func = self._get_resource(_function.Function, function)
        function_urn = func.func_urn.rpartition(":")[0]
        return self._get(
            _log.Log,
            function_urn=function_urn,
            requires_id=False
        )

    def enable_lts_log(self):
        """Enable log reporting to LTS.

        :returns: The created Log instance.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.log.Log`
        """
        base_path = '/fgs/functions/enable-lts-logs'
        return self._create(
            _log.Log, base_path=base_path
        )

    # ======== Templates Methods ========

    def get_template(self, template_id):
        """Get one template by ID.

        :param template_id: id of template
        :returns: instance of
            :class:`~otcextensions.sdk.function_graph.v2.template.Template`
        """
        return self._get(
            _t.Template,
            template_id=template_id,
            requires_id=False
        )

    # ======== Reserved Instances Methods ========

    def update_instances_number(self, function, **attrs):
        """Update a number of reserved instances from attributes.

        :param function: The URN or instance of the Function to update
            event in.
        :param dict attrs: Keyword arguments to update a reserved instances.
        :returns: The updated ReservedInstance instance.
        """
        function = self._get_resource(_function.Function, function)
        function_urn = function.func_urn.rpartition(":")[0]
        return self._update(
            _r.ReservedInstance, function_urn=function_urn, **attrs
        )

    def reserved_instances_config(self, **query):
        """List all reserved instances of a function.

        :returns: A generator of ReservedInstance instances.
        """
        base_path = "/fgs/functions/reservedinstanceconfigs"
        return self._list(_r.ReservedInstance, base_path=base_path, **query)

    def reserved_instances(self, **query):
        """List all query the number of instances reserved for a function.

        :returns: A generator of ReservedInstance instances.
        """
        base_path = "/fgs/functions/reservedinstances"
        return self._list(_r.ReservedInstance, base_path=base_path, **query)

    # ======== Import/Export Methods ========

    def export_function(self, function, **query):
        """Export a function.

        :param function: The URN or instance of the Function to export.
        :returns: instance of :class:
            `~otcextensions.sdk.function_graph.v2.export_function.Export`
        """
        function = self._get_resource(_function.Function, function)
        export = self._get_resource(_export.Export, "")
        return export._export(self, function, **query)

    def import_function(self, **attrs):
        """Import a function.

        :param dict attrs: Keyword arguments to import Function.
        :returns: The Import instance.
        """
        return self._create(
            _import.Import, **attrs
        )

    # ======== Trigger Methods ========

    def create_trigger(self, function, **attrs):
        """Create a new function trigger from attributes.

        :param dict attrs: Keyword arguments to create a Function.
        :returns: The created Trigger instance.
        :rtype: :class:`~otcextensions.sdk.function_graph.v2.trigger.Trigger`
        """
        function = self._get_resource(_function.Function, function)
        function_urn = function.func_urn.rpartition(":")[0]
        base_path = f'/fgs/triggers/{function_urn}'
        return self._create(_trigger.Trigger, base_path=base_path, **attrs)

    def delete_trigger(
            self, function_urn, trigger_type_code, trigger_id,
            ignore_missing=True
    ):
        """Delete a function trigger.

        :param trigger_id: Trigger ID.
        :param trigger_type_code: Trigger type code.
        :param function_urn: Function URN
        :param ignore_missing:
        :returns: ``None``
        """
        trigger = self._get_resource(_trigger.Trigger, "")
        return trigger._delete_trigger(
            self, function_urn, trigger_type_code, trigger_id
        )

    def delete_all_triggers(self, function_urn, ignore_missing=True):
        """Delete all function triggers.

        :param function_urn: Function URN
        :param ignore_missing:
        :returns: ``None``
        """
        function_urn = function_urn.rpartition(":")[0]
        trigger = self._get_resource(_trigger.Trigger, "")
        return trigger._delete_triggers(self, function_urn)

    def triggers(self, function_urn):
        """List all triggers of a function.

        :param function_urn: Function URN
        :returns: A generator of Trigger instances.
        """
        function_urn = function_urn.rpartition(":")[0]
        return self._list(_trigger.Trigger, function_urn=function_urn)

    def get_trigger(self, function_urn, trigger_type_code, trigger_id):
        """Get one trigger.

        :param trigger_id: Trigger ID.
        :param trigger_type_code: Trigger type code.
        :param function_urn: Function URN.
        :returns: instance of
            :class:`~otcextensions.sdk.function_graph.v2.trigger.Trigger`
        """
        function_urn = function_urn.rpartition(":")[0]
        base_path = (f'/fgs/triggers/{function_urn}'
                     f'/{trigger_type_code}'
                     f'/{trigger_id}')
        return self._get(
            _trigger.Trigger,
            base_path=base_path,
            requires_id=False
        )

    def update_trigger(
            self, function_urn, trigger_type_code, trigger_id, **attrs
    ):
        """Update a number of reserved instances from attributes.

        :param function_urn: Function URN.
        :param trigger_id: Trigger ID.
        :param trigger_type_code: Trigger type code.
        :param dict attrs: Keyword arguments to update a trigger.
        :returns: The updated Trigger instance.
        """
        function_urn = function_urn.rpartition(":")[0]
        base_path = (f'/fgs/triggers/{function_urn}/{trigger_type_code}'
                     f'/{trigger_id}')
        return self._update(
            _trigger.Trigger, base_path=base_path, **attrs
        )

    # ======== Asynchronous Invocation Methods ========

    def async_notifications(self, function):
        """List asynchronous invocation setting of a function version.

        :param function: Function instance or function URN
        :returns: A generator of Notification instances.
        """
        function = self._get_resource(_function.Function, function)
        if function.id is not None:
            function_urn = function.id.rpartition(":")[0]
        else:
            function_urn = function.func_urn.rpartition(":")[0]
        return self._list(_async.Notification, function_urn=function_urn)

    def configure_async_notification(
            self, function, **attrs
    ):
        """Configure asynchronous execution notification for a function.

        :param function: Function instance or function URN
        :param dict attrs: Keyword arguments to update a notifications
            settings.
        :returns: The updated Notification instance.
        """
        function = self._get_resource(_function.Function, function)
        if function.id is not None:
            function_urn = function.id.rpartition(":")[0]
        else:
            function_urn = function.func_urn.rpartition(":")[0]
        return self._create(
            _async.Notification, function_urn=function_urn, **attrs
        )

    def delete_async_notification(self, function, ignore_missing=True):
        """Delete asynchronous execution notification for a function.

        :param function: The URN or instance of the Function to delete
            alias from.
        :param ignore_missing: When False,
            `openstack.exceptions.ResourceNotFound`
            will be raised when the tag does not exist.
            When True, no exception will be set when attempting
            to delete a nonexistent event.
        :returns: ``None``
        """
        function = self._get_resource(_function.Function, function)
        if function.id is not None:
            function_urn = function.id.rpartition(":")[0]
        else:
            function_urn = function.func_urn.rpartition(":")[0]
        return self._delete(
            _async.Notification,
            function_urn=function_urn, ignore_missing=ignore_missing,
        )

    def all_versions_async_notifications(self, function, **query):
        """List asynchronous invocation setting of all function versions.

        :param function: Function instance or function URN
        :returns: A generator of Notification instances.
        """
        function = self._get_resource(_function.Function, function)
        if function.id is not None:
            function_urn = function.id
        else:
            function_urn = function.func_urn
        return self._list(
            _async.Notification,
            function_urn=function_urn,
            **query)

    def async_invocation_requests(self, function, **query):
        """Get asynchronous invocation requests of a function.

        :param function: Function instance or function URN
        :returns: instance of Requests
        """
        function = self._get_resource(_function.Function, function)
        if function.id is not None:
            function_urn = function.id.rpartition(":")[0]
        else:
            function_urn = function.func_urn.rpartition(":")[0]
        return self._list(
            _async.Requests,
            function_urn=function_urn,
            **query
        )

    def stop_async_invocation_request(self, function, **attrs):
        """Stop asynchronous invocation request of a function.

        :param function: Function instance or function URN
        :returns: instance of Requests
        """
        function = self._get_resource(_function.Function, function)
        stop = self._get_resource(_async.Requests, "")
        if function.id is not None:
            function_urn = function.id.rpartition(":")[0]
        else:
            function_urn = function.func_urn.rpartition(":")[0]
        return stop._stop(self, function_urn=function_urn, **attrs)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DCS Instance v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

STATUS_VALUES = ['CREATING', 'CREATEFAILED', 'RUNNING', 'ERROR', 'STARTING',
                 'RESTARTING', 'CLOSING', 'CLOSED', 'EXTENDING']

PRODUCT_ID_VALUES = ['OTC_DCS_SINGLE', 'OTC_DCS_CL', 'OTC_DCS_MS']

_formatters = {
}


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListInstance(command.Lister):
    _description = _('List DCS Instances')
    columns = ('id', 'name', 'engine', 'status', 'error_code')

    def get_parser(self, prog_name):
        parser = super(ListInstance, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Limit number of records to return.')
        )
        parser.add_argument(
            '--start',
            metavar='<start>',
            type=int,
            help=_('Start number for querying.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('DCS instance name.')
        )
        parser.add_argument(
            '--status',
            metavar='{' + ','.join(STATUS_VALUES) + '}',
            type=lambda s: s.upper(),
            choices=STATUS_VALUES,
            help=_('DCS instance status value to query.')
        )
        parser.add_argument(
            '--include_failure',
            action='store_true',
            help=_('An indicator of whether the number of DCS instances '
                   'that failed to be created will be returned to the API '
                   'caller.')
        )
        parser.add_argument(
            '--exact_match',
            action='store_true',
            help=_('An indicator of whether to perform an exact or fuzzy '
                   'match based on instance name.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcs

        query = {}
        if parsed_args.limit:
            query['limit'] = parsed_args.limit
        if parsed_args.start:
            query['start'] = parsed_args.start
        if parsed_args.name:
            query['name'] = parsed_args.name
        if parsed_args.status:
            query['status'] = parsed_args.status
        if parsed_args.include_failure:
            query['includeFailure'] = parsed_args.include_failure
        if parsed_args.exact_match:
            query['exactMatchName'] = parsed_args.exact_match

        data = client.instances(
            **query
        )

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns,
                 ) for s in data))
        return table


class DeleteInstance(command.Command):
    _description = _('Delete DCS Instance')

    def get_parser(self, prog_name):
        parser = super(DeleteInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            nargs='+',
            help=_('Name or ID of the instance to delete.')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.instance:
            client = self.app.client_manager.dcs
            for instance in parsed_args.instance:
                client.delete_instance(instance=instance)


class ShowInstance(command.ShowOne):
    _description = _('Show the details of a single instance')

    def get_parser(self, prog_name):
        parser = super(ShowInstance, self).get_parser(prog_name)

        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Name or UUID of the instance.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dcs

        obj = client.find_instance(
            name_or_id=parsed_args.instance)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class CreateInstance(command.ShowOne):
    _description = _('Create a single DCS instance')

    def get_parser(self, prog_name):
        parser = super(CreateInstance, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='<name>',
            required=True,
            help=_('DCS instance name. An instance name is a string of '
                   '4-64 characters that contain letters, digits, underscores '
                   '(_), and hyphens (-). An instance name must start '
                   'with letters.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Brief description of the DCS instance. A brief '
                   'description supports up to 1024 characters. NOTE "\\" '
                   'is defined as an escape character in the queue '
                   'description. If you need to enter a backward slash (\\) '
                   'or a double quotation mark (") in the queue description, '
                   'enter \\ or \\\"')
        )
        parser.add_argument(
            '--engine',
            metavar='<engine>',
            default='Redis',
            help=_('Cache engine, which is Redis.')
        )
        parser.add_argument(
            '--engine_version',
            metavar='<version>',
            default='3.0.7',
            help=_('Cache engine version, which is 3.0.7.')
        )
        parser.add_argument(
            '--capacity',
            metavar='<size>',
            type=int,
            required=True,
            choices=[1, 2, 4, 8, 16, 32, 64, 128, 256, 512],
            help=_('Cache capacity. Unit: GB. For a single-node DCS instance '
                   'and master/standby DCS instance, the cache capacity can '
                   'be: 1, 2, 4, 8, 16, 32, or 64 GB. For a DCS instance in '
                   'cluster mode, the cache capacity can be 64, 128, 256, '
                   'or 512 GB.')
        )
        parser.add_argument(
            '--password',
            metavar='<password>',
            required=True,
            help=_('Password of a DCS instance. Password complexity '
                   'requirements:\n'
                   '* A string of 8-32 characters.\n'
                   '* Contains at least three types of the following '
                   'characters:\n'
                   '* Uppercase letters\n'
                   '* Lowercase letters\n'
                   '* Digits\n'
                   '* Special characters, such as '
                   '~!@#$%%^&*()-_=+\\|[{}]:\'",<.>/?')
        )
        parser.add_argument(
            '--vpc_id',
            metavar='<id>',
            required=True,
            help=_('Tenant\'s Virtual Private Cloud (VPC) ID.')
        )
        parser.add_argument(
            '--security_group_id',
            metavar='<id>',
            required=True,
            help=_('Tenant\'s security group ID.')
        )
        parser.add_argument(
            '--subnet_id',
            metavar='<id>',
            required=True,
            help=_('Subnet ID.')
        )
        parser.add_argument(
            '--az',
            metavar='<az>',
            required=True,
            help=_('Availability zone.')
        )
        parser.add_argument(
            '--product_id',
            metavar='{' + ','.join(PRODUCT_ID_VALUES) + '}',
            type=lambda s: s.upper(),
            choices=PRODUCT_ID_VALUES,
            required=True,
            help=_('Product ID used to differentiate DCS instance types. '
                   'Values:\n'
                   '* OTC_DCS_SINGLE: indicates a single-node DCS instance.\n'
                   '* OTC_DCS_MS: indicates a master/standby DCS instance.\n'
                   '* OTC_DCS_CL: indicates a DCS instance in cluster mode.')
        )
        parser.add_argument(
            '--backup_policy',
            metavar='<json>',
            help=_('Backup policy JSON.\n'
                   '{\n'
                   '"save_days" - retention time [1-7]\n'
                   '"backup_type" - [auto,manual]\n'
                   '"periodical_backup_plan": {\n'
                   '"begin_at" - time when backup starts\n'
                   '"period_type" - [weekly]\n'
                   '"backup_at" - array of days [1-7]\n'
                   '}\n'
                   '}')
        )
        parser.add_argument(
            '--maintain_begin',
            metavar='<HH:mm:ss>',
            help=_('Time at which the maintenance time window starts.\n'
                   'The start time must be set to 22:00:00, 02:00:00,'
                   '06:00:00, 10:00:00, 14:00:00, or 18:00:00.\n'
                   'Parameters maintain_begin and maintain_end must be '
                   'set in pairs. If parameter maintain_start is left blank, '
                   'parameter maintain_end is also blank. In this case, the '
                   'system automatically allocates the default start time '
                   '02:00:00.')
        )
        parser.add_argument(
            '--maintain_end',
            metavar='<HH:mm:ss>',
            help=_('Time at which the maintenance time window starts.\n'
                   'The end time is four hours later than the start time. '
                   'For example, if the start time is 22:00:00, the end time '
                   'is 02:00:00.\n'
                   'Parameters maintain_begin and maintain_end must be '
                   'set in pairs. If parameter maintain_end is left blank, '
                   'parameter maintain_start is also blank. In this case, the '
                   'system automatically allocates the default end time '
                   '06:00:00.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dcs

        attrs = {}

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.engine:
            attrs['engine'] = parsed_args.engine
        if parsed_args.engine_version:
            attrs['engine_version'] = parsed_args.engine_version
        if parsed_args.capacity:
            attrs['capacity'] = parsed_args.capacity
        if parsed_args.password:
            attrs['password'] = parsed_args.password
        if parsed_args.vpc_id:
            attrs['vpc_id'] = parsed_args.vpc_id
        if parsed_args.security_group_id:
            attrs['security_group_id'] = parsed_args.security_group_id
        if parsed_args.subnet_id:
            attrs['subnet_id'] = parsed_args.subnet_id
        if parsed_args.az:
            attrs['az'] = parsed_args.az
        if parsed_args.product_id:
            attrs['product_id'] = parsed_args.product_id
        if parsed_args.backup_policy:
            attrs['backup_policy'] = parsed_args.backup_policy
        if parsed_args.maintain_begin and parsed_args.maintain_end:
            attrs['maintain_begin'] = parsed_args.maintain_begin
            attrs['maintain_end'] = parsed_args.maintain_end

        obj = client.create_instance(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class SetInstance(command.ShowOne):
    _description = _('Update a single DCS instance')

    def get_parser(self, prog_name):
        parser = super(SetInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or Name of the instance to modify')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('DCS instance name. An instance name is a string of '
                   '4-64 characters that contain letters, digits, underscores '
                   '(_), and hyphens (-). An instance name must start '
                   'with letters.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Brief description of the DCS instance. A brief '
                   'description supports up to 1024 characters. NOTE "\\" '
                   'is defined as an escape character in the queue '
                   'description. If you need to enter a backward slash (\\) '
                   'or a double quotation mark (") in the queue description, '
                   'enter \\ or \\\"')
        )
        parser.add_argument(
            '--security_group_id',
            metavar='<id>',
            help=_('Tenant\'s security group ID.')
        )
        parser.add_argument(
            '--backup_policy',
            metavar='<json>',
            help=_('Backup policy JSON.\n'
                   '{\n'
                   '"save_days" - retention time [1-7]\n'
                   '"backup_type" - [auto,manual]\n'
                   '"periodical_backup_plan": {\n'
                   '"begin_at" - time when backup starts\n'
                   '"period_type" - [weekly]\n'
                   '"backup_at" - array of days [1-7]\n'
                   '}\n'
                   '}')
        )
        parser.add_argument(
            '--maintain_begin',
            metavar='<HH:mm:ss>',
            help=_('Time at which the maintenance time window starts.\n'
                   'The start time must be set to 22:00:00, 02:00:00,'
                   '06:00:00, 10:00:00, 14:00:00, or 18:00:00.\n'
                   'Parameters `maintain_begin` and `maintain_end` must be '
                   'set in pairs. If parameter `maintain_start` is left '
                   'blank, parameter `maintain_end` is also blank. In this '
                   'case, the system automatically allocates the default '
                   'start time 02:00:00.')
        )
        parser.add_argument(
            '--maintain_end',
            metavar='<HH:mm:ss>',
            help=_('Time at which the maintenance time window starts.\n'
                   'The end time is four hours later than the start time. '
                   'For example, if the start time is 22:00:00, the end time '
                   'is 02:00:00.\n'
                   'Parameters `maintain_begin` and `maintain_end` must be '
                   'set in pairs. If parameter `maintain_end` is left blank, '
                   'parameter `maintain_start` is also blank. In this case, '
                   'the system automatically allocates the default end time '
                   '06:00:00.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dcs

        attrs = {}

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.security_group_id:
            attrs['security_group_id'] = parsed_args.security_group_id
        if parsed_args.backup_policy:
            attrs['backup_policy'] = parsed_args.backup_policy
        if parsed_args.maintain_begin and parsed_args.maintain_end:
            attrs['maintain_begin'] = parsed_args.maintain_begin
            attrs['maintain_end'] = parsed_args.maintain_end

        obj = client.update_instance(instance=parsed_args.instance, **attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class ExtendInstance(command.ShowOne):
    _description = _('Extend capacity of a single DCS instance')

    def get_parser(self, prog_name):
        parser = super(ExtendInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or Name of the instance to modify')
        )
        parser.add_argument(
            '--capacity',
            metavar='<size>',
            type=int,
            required=True,
            choices=[1, 2, 4, 8, 16, 32, 64, 128, 256, 512],
            help=_('Value in GB for the new instance capacity.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dcs

        inst = client.find_instance(
            name_or_id=parsed_args.instance,
            ignore_missing=False)

        obj = client.extend_instance(
            instance=inst.id,
            capacity=parsed_args.capacity)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class _OperationInstance(command.ShowOne):
    def get_parser(self, prog_name):
        parser = super(_OperationInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or Name of the instance to modify')
        )
        return parser


class StopInstance(_OperationInstance):
    _description = _('Stop a single DCS instance')

    def take_action(self, parsed_args):

        client = self.app.client_manager.dcs

        obj = client.stop_instance(instance=parsed_args.instance)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class StartInstance(_OperationInstance):
    _description = _('Start a single DCS instance')

    def take_action(self, parsed_args):

        client = self.app.client_manager.dcs

        obj = client.start_instance(instance=parsed_args.instance)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class RestartInstance(_OperationInstance):
    _description = _('Restart a single DCS instance')

    def take_action(self, parsed_args):

        client = self.app.client_manager.dcs

        obj = client.restart_instance(instance=parsed_args.instance)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class ChangePasswordInstance(command.ShowOne):
    _description = _('Change a password of a single DCS instance')

    def get_parser(self, prog_name):
        parser = super(ChangePasswordInstance, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or Name of the instance to modify')
        )
        parser.add_argument(
            '--current_password',
            metavar='<pwd>',
            help=_('Current instance password')
        )
        parser.add_argument(
            '--new_password',
            metavar='<pwd>',
            help=_('New password of a DCS instance. Password complexity '
                   'requirements:\n'
                   '* A string of 8-32 characters.\n'
                   '* Contains at least three types of the following '
                   'characters:\n'
                   '* Uppercase letters\n'
                   '* Lowercase letters\n'
                   '* Digits\n'
                   '* Special characters, such as '
                   '~!@#$%%^&*()-_=+\\|[{}]:\'",<.>/?')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dcs

        obj = client.change_instance_password(
            instance=parsed_args.instance,
            current_password=parsed_args.current_password,
            new_password=parsed_args.new_password)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from unittest.mock import call

import mock
from openstackclient.tests.unit import utils as tests_utils
from osc_lib import exceptions

from otcextensions.osclient.vpcep.v1 import service
from otcextensions.tests.unit.osclient.vpcep.v1 import fakes


class TestListServices(fakes.TestVpcep):

    objects = fakes.FakeService.create_multiple(3)

    column_list_headers = (
        'Id',
        'Service Name',
        'Service Type',
        'Server Type',
        'Connection Count',
        'Status',
    )

    columns = (
        'id',
        'service_name',
        'service_type',
        'server_type',
        'connection_count',
        'status',
    )

    data = []

    for s in objects:
        data.append(
            (
                s.id,
                s.service_name,
                s.service_type,
                s.server_type,
                s.connection_count,
                s.status,
            )
        )

    def setUp(self):
        super(TestListServices, self).setUp()

        self.cmd = service.ListServices(self.app, None)

        self.client.services = mock.Mock()
        self.client.api_mock = self.client.services

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--id',
            '1',
            '--name',
            '2',
            '--status',
            '3',
            '--sort-key',
            'created_at',
            '--sort-dir',
            'desc',
            '--limit',
            '6',
            '--offset',
            '7',
        ]

        verifylist = [
            ('id', '1'),
            ('name', '2'),
            ('status', '3'),
            ('sort_key', 'created_at'),
            ('sort_dir', 'desc'),
            ('limit', 6),
            ('offset', 7),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            id='1',
            name='2',
            status='3',
            sort_key='created_at',
            sort_dir='desc',
            limit=6,
            offset=7,
        )


class TestCreateService(fakes.TestVpcep):

    _data = fakes.FakeService.create_one()

    columns = (
        'created_at',
        'id',
        'is_approval_enabled',
        'pool_id',
        'port_id',
        'ports',
        'project_id',
        'router_id',
        'server_type',
        'service_name',
        'service_type',
        'status',
    )

    data = fakes.gen_data(_data, columns, formatters=service._formatters)

    def setUp(self):
        super(TestCreateService, self).setUp()

        self.cmd = service.CreateService(self.app, None)
        self.client.create_service = mock.Mock(
            return_value=fakes.FakeService.create_one()
        )

    def test_create(self):
        arglist = [
            'test-endpoint-service',
            '--port-id',
            'test-port-uuid',
            '--pool-id',
            'test-pool-uuid',
            '--router-id',
            'test-router-uuid',
            '--disable-approval',
            '--server-type',
            'VM',
            '--service-type',
            'Interface',
            '--ports',
            'client_port=80,server_port=80,protocol=TCP',
            '--tags',
            'key=tag-key,value=tag-value',
            '--tcp-proxy',
            'open',
        ]
        verifylist = [
            ('name', 'test-endpoint-service'),
            ('port_id', 'test-port-uuid'),
            ('pool_id', 'test-pool-uuid'),
            ('vpc_id', 'test-router-uuid'),
            ('disable_approval', True),
            ('server_type', 'VM'),
            ('service_type', 'interface'),
            (
                'ports',
                [
                    {
                        'client_port': '80',
                        'server_port': '80',
                        'protocol': 'TCP',
                    }
                ],
            ),
            ('tags', [{'key': 'tag-key', 'value': 'tag-value'}]),
            ('tcp_proxy', 'open'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            'service_name': 'test-endpoint-service',
            'port_id': 'test-port-uuid',
            'pool_id': 'test-pool-uuid',
            'vpc_id': 'test-router-uuid',
            'approval_enabled': False,
            'server_type': 'VM',
            'service_type': 'interface',
            'ports': [
                {'client_port': 80, 'server_port': 80, 'protocol': 'TCP'}
            ],
            'tags': [{'key': 'tag-key', 'value': 'tag-value'}],
            'tcp_proxy': 'open',
        }

        self.client.create_service.assert_called_with(**attrs)
        self.assertEqual(self.columns, columns)


class TestShowService(fakes.TestVpcep):

    _data = fakes.FakeService.create_one()

    columns = (
        'created_at',
        'id',
        'is_approval_enabled',
        'pool_id',
        'port_id',
        'ports',
        'project_id',
        'router_id',
        'server_type',
        'service_name',
        'service_type',
        'status',
    )

    data = fakes.gen_data(_data, columns, formatters=service._formatters)

    def setUp(self):
        super(TestShowService, self).setUp()

        self.cmd = service.ShowService(self.app, None)

        self.client.find_service = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = [
            self._data.id,
        ]

        verifylist = [
            ('service', self._data.id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_service.assert_called_with(self._data.id)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            'non-existing-service',
        ]

        verifylist = [
            ('service', 'non-existing-service'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_service = mock.Mock(side_effect=find_mock_result)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.find_service.assert_called_with('non-existing-service')


class TestUpdateService(fakes.TestVpcep):

    _data = fakes.FakeService.create_one()

    columns = (
        'created_at',
        'id',
        'is_approval_enabled',
        'pool_id',
        'port_id',
        'ports',
        'project_id',
        'router_id',
        'server_type',
        'service_name',
        'service_type',
        'status',
    )

    data = fakes.gen_data(_data, columns, formatters=service._formatters)

    def setUp(self):
        super(TestUpdateService, self).setUp()

        self.cmd = service.UpdateService(self.app, None)

        self.client.find_service = mock.Mock(return_value=self._data)
        self.client.update_service = mock.Mock(return_value=self._data)

    def test_update(self):
        arglist = [
            self._data.name,
            '--name',
            'test-endpoint-service',
            '--ports',
            'client_port=80,server_port=80,protocol=TCP',
            '--port-id',
            'test-port-uuid',
            '--tcp-proxy',
            'open',
            '--enable-approval',
        ]
        verifylist = [
            ('service', self._data.name),
            ('service_name', 'test-endpoint-service'),
            (
                'ports',
                [
                    {
                        'client_port': '80',
                        'server_port': '80',
                        'protocol': 'TCP',
                    }
                ],
            ),
            ('port_id', 'test-port-uuid'),
            ('tcp_proxy', 'open'),
            ('enable_approval', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_service.assert_called_with(self._data.name)

        attrs = {
            'service_name': 'test-endpoint-service',
            'port_id': 'test-port-uuid',
            'ports': [
                {'client_port': 80, 'server_port': 80, 'protocol': 'TCP'}
            ],
            'tcp_proxy': 'open',
            'approval_enabled': True,
        }
        self.client.update_service.assert_called_with(self._data, **attrs)
        self.assertEqual(self.columns, columns)


class TestDeleteService(fakes.TestVpcep):

    _data = fakes.FakeService.create_multiple(2)

    def setUp(self):
        super(TestDeleteService, self).setUp()

        self.client.delete_service = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = service.DeleteService(self.app, None)

    def test_delete(self):
        arglist = [
            self._data[0].name,
        ]

        verifylist = [
            ('service', [self._data[0].name]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.client.find_service = mock.Mock(return_value=self._data[0])

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_service.assert_called_with(self._data[0].id)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for data in self._data:
            arglist.append(data.name)

        verifylist = [
            ('service', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = self._data
        self.client.find_service = mock.Mock(side_effect=find_mock_result)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for data in self._data:
            calls.append(call(data.id))
        self.client.delete_service.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._data[0].name,
            'non-existing-service',
        ]
        verifylist = [
            ('service', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = [self._data[0], exceptions.CommandError]
        self.client.find_service = mock.Mock(side_effect=find_mock_result)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual(
                '1 of 2 Vpc Endpoint Services(s) failed to delete.', str(e)
            )

        self.client.find_service.assert_any_call(self._data[0].name)
        self.client.find_service.assert_any_call('non-existing-service')
        self.client.delete_service.assert_called_once_with(self._data[0].id)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
List all CloudEye event data
"""
import openstack


openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


query = {
    'namespace': 'SYS.ECS',
    'type': 'instance_host_info',
    'dim.0': 'instance_id,6e83e6e7-3bf4-4b5b-b390-e80447ef1234',  # key, value
    'from': '1596067200',  # unix timestamp in ms
    'to': '1597929178'  # unix timestamp in ms
}

for data in conn.ces.event_data(**query):
    print(data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import app


EXAMPLE_APP = {
    'id': 'app-12345',
    'gateway_id': 'gateway-67890',
    'name': 'Test App',
    'remark': 'This is a test app',
    'app_key': 'key-abcdef',
    'app_secret': 'secret-12345',
    'creator': 'user-1',
    'update_time': '2025-02-07T12:30:00Z',
    'register_time': '2025-02-07T12:00:00Z',
    'status': 1,
    'app_type': 'web',
    'roma_app_type': 'roma-type-1',
    'bind_num': 10
}


class TestApp(base.TestCase):

    def test_basic(self):
        sot = app.App()
        self.assertEqual('/apigw/instances/%(gateway_id)s/apps', sot.base_path)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertEqual('apps', sot.resources_key)

    def test_make_it(self):
        sot = app.App(**EXAMPLE_APP)
        self.assertEqual(EXAMPLE_APP['id'], sot.id)
        self.assertEqual(EXAMPLE_APP['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE_APP['name'], sot.name)
        self.assertEqual(EXAMPLE_APP['remark'], sot.remark)
        self.assertEqual(EXAMPLE_APP['app_key'], sot.app_key)
        self.assertEqual(EXAMPLE_APP['app_secret'], sot.app_secret)
        self.assertEqual(EXAMPLE_APP['creator'], sot.creator)
        self.assertEqual(EXAMPLE_APP['update_time'], sot.update_time)
        self.assertEqual(EXAMPLE_APP['register_time'], sot.register_time)
        self.assertEqual(EXAMPLE_APP['status'], sot.status)
        self.assertEqual(EXAMPLE_APP['app_type'], sot.app_type)
        self.assertEqual(EXAMPLE_APP['roma_app_type'], sot.roma_app_type)
        self.assertEqual(EXAMPLE_APP['bind_num'], sot.bind_num)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import api_auth


EXAMPLE_API_AUTH_RESULT = {
    'status': 'AUTHORIZED',
    'error_msg': '',
    'error_code': '',
    'api_name': 'Test API',
    'app_name': 'Test App'
}

EXAMPLE_API_AUTH_INFO = {
    'gateway_id': 'gateway-67890',
    'env_id': 'env-1',
    'app_ids': ['app-12345'],
    'api_ids': ['api-98765'],
    'api_id': 'api-98765',
    'api_name': 'Test API',
    'group_name': 'Group A',
    'api_type': 1,
    'api_remark': 'Test API remark',
    'app_name': 'Test App',
    'app_remark': 'Test App remark',
    'app_type': 'web',
    'app_creator': 'creator-1',
    'publish_id': 'publish-999',
    'group_id': 'group-111',
    'auth_result': EXAMPLE_API_AUTH_RESULT,
    'auth_time': '2025-02-07T12:00:00Z',
    'id': 'auth-abc',
    'app_id': 'app-12345',
    'auth_role': 'admin',
    'auth_tunnel': 'default',
    'auth_whitelist': ['192.168.0.1'],
    'auth_blacklist': ['10.0.0.1'],
    'visit_params': 'param1=value1',
    'visit_param': 'param1',
    'roma_app_type': 'roma-type-1',
    'env_name': 'production',
    'run_env_name': 'prod'
}


class TestApiAuthResult(base.TestCase):

    def test_make_it(self):
        sot = api_auth.ApiAuthResult(**EXAMPLE_API_AUTH_RESULT)
        self.assertEqual(EXAMPLE_API_AUTH_RESULT['status'], sot.status)
        self.assertEqual(EXAMPLE_API_AUTH_RESULT['error_msg'], sot.error_msg)
        self.assertEqual(EXAMPLE_API_AUTH_RESULT['error_code'], sot.error_code)
        self.assertEqual(EXAMPLE_API_AUTH_RESULT['api_name'], sot.api_name)
        self.assertEqual(EXAMPLE_API_AUTH_RESULT['app_name'], sot.app_name)


class TestApiAuthInfo(base.TestCase):

    def test_basic(self):
        sot = api_auth.ApiAuthInfo()
        self.assertEqual('auths', sot.resources_key)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)

    def test_make_it(self):
        sot = api_auth.ApiAuthInfo(**EXAMPLE_API_AUTH_INFO)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['gateway_id'],
                         sot.gateway_id)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['env_id'],
                         sot.env_id)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['app_ids'],
                         sot.app_ids)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['api_ids'],
                         sot.api_ids)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['api_id'],
                         sot.api_id)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['api_name'],
                         sot.api_name)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['group_name'],
                         sot.group_name)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['api_type'],
                         sot.api_type)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['api_remark'],
                         sot.api_remark)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['app_name'],
                         sot.app_name)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['app_remark'],
                         sot.app_remark)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['app_type'],
                         sot.app_type)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['app_creator'],
                         sot.app_creator)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['publish_id'],
                         sot.publish_id)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['group_id'],
                         sot.group_id)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['auth_time'],
                         sot.auth_time)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['id'], sot.id)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['app_id'],
                         sot.app_id)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['auth_role'],
                         sot.auth_role)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['auth_tunnel'],
                         sot.auth_tunnel)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['auth_whitelist'],
                         sot.auth_whitelist)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['auth_blacklist'],
                         sot.auth_blacklist)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['visit_params'],
                         sot.visit_params)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['visit_param'],
                         sot.visit_param)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['roma_app_type'],
                         sot.roma_app_type)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['env_name'],
                         sot.env_name)
        self.assertEqual(EXAMPLE_API_AUTH_INFO['run_env_name'],
                         sot.run_env_name)

        self.assertEqual(EXAMPLE_API_AUTH_RESULT['status'],
                         sot.auth_result.status)
        self.assertEqual(EXAMPLE_API_AUTH_RESULT['api_name'],
                         sot.auth_result.api_name)


class TestApiAuth(base.TestCase):

    def test_basic(self):
        sot = api_auth.ApiAuth()
        self.assertEqual('apis', sot.resources_key)
        self.assertFalse(sot.allow_create)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import random
import uuid
import time

import mock

from openstackclient.tests.unit import utils
from otcextensions.tests.unit.osclient import test_base

from otcextensions.sdk.dns.v2 import zone
from otcextensions.sdk.dns.v2 import nameserver
from otcextensions.sdk.dns.v2 import recordset
from otcextensions.sdk.dns.v2 import floating_ip


def gen_data(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(getattr(data, attr, '') for attr in columns)


class TestDNS(utils.TestCommand):

    def setUp(self):
        super(TestDNS, self).setUp()

        self.app.client_manager.dns = mock.Mock()
        self.client = self.app.client_manager.dns


class FakeZone(test_base.Fake):
    """Fake one or more Zone"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': uuid.uuid4().hex,
            'action': uuid.uuid4().hex,
            'created_at': uuid.uuid4().hex,
            'description': uuid.uuid4().hex,
            'email': uuid.uuid4().hex,
            'pool_id': uuid.uuid4().hex,
            'record_num': random.randint(1, 600),
            'router': {
                'router_id': uuid.uuid4().hex,
                'router_region': uuid.uuid4().hex,
                'status': uuid.uuid4().hex,
            },
            'serial': random.randint(1, 600),
            'status': uuid.uuid4().hex,
            'ttl': random.randint(1, 600),
            'updated_at': time.time() * 1000,
            'zone_type': uuid.uuid4().hex,
        }
        obj = zone.Zone.existing(**object_info)
        return obj


class FakeNameserver(test_base.Fake):
    """Fake one or more Zone"""

    @classmethod
    def generate(cls):
        object_info = {
            'zone_id': 'id-' + uuid.uuid4().hex,
            'address': uuid.uuid4().hex,
            'hostname': uuid.uuid4().hex,
            'priority': random.randint(1, 600),
        }
        obj = nameserver.NameServer.existing(**object_info)
        return obj


class FakeRecordset(test_base.Fake):
    """Fake one or more recordset"""

    @classmethod
    def generate(cls):
        object_info = {
            'zone_id': 'id-' + uuid.uuid4().hex,
            'name': uuid.uuid4().hex,
            'description': uuid.uuid4().hex,
            'status': uuid.uuid4().hex,
            'ttl': random.randint(1, 600),
            'type': random.choice(['A', 'AAAA', 'SRV']),
            'records': list(random.randint(1, 600) for i in range(0, 5))
        }
        obj = recordset.Recordset.existing(**object_info)
        return obj


class FakeFloatingIP(test_base.Fake):
    """Fake one or more recordset"""

    @classmethod
    def generate(cls):
        object_info = {
            'address': uuid.uuid4().hex,
            'floating_ip_id': uuid.uuid4().hex,
            'region': uuid.uuid4().hex,
            'id': uuid.uuid4().hex,
            'ptrdname': uuid.uuid4().hex,
            'description': uuid.uuid4().hex,
            'ttl': random.randint(1, 600),
        }
        obj = floating_ip.FloatingIP.existing(**object_info)
        return obj



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import random
import uuid

import mock

from otcextensions.sdk.vlb.v3 import listener
from otcextensions.sdk.vlb.v3 import load_balancer
from otcextensions.tests.unit.osclient import test_base


def generate_eips_list():
    """Generate random list of vault UUIDs"""
    eips = [{"eip_id": 'eip-uuid-1',
             "eip_address": 'eip-address-1',
             "ip_version": 'ip-version-1'},
            {"eip_id": 'eip-uuid-2',
             "eip_address": 'eip-address-2',
             "ip_version": 'ip-version-2'}
            ]
    return eips


def generate_floating_ips_list():
    """Generate random list of vault UUIDs"""
    floating_ips = [{"publicip_id": "publicip-id-1",
                     "publicip_address": "publicip-address-1",
                     "ip_version": "ip-version-1"},
                    {"publicip_id": "publicip-id-2",
                     "publicip_address": "publicip-address-2",
                     "ip_version": "ip-version-2"}
                    ]
    return floating_ips


def generate_listeners_list():
    """Generate random list of vault UUIDs"""
    listeners = [{"id": 'listener-id-1'},
                 {"id": 'listener-id-2'}]
    return listeners


def generate_pools_list():
    """Generate random list of vault UUIDs"""
    pools = [{"id": 'pool-id-1'},
             {"id": 'pool-id-2'}]
    return pools


def generate_network_ids_list():
    """Generate random list of vault UUIDs"""
    network_ids = []
    random_int = random.randint(1, 10)
    while random_int > 0:
        network_ids.append(uuid.uuid4().hex)
        random_int -= 1
    return network_ids


def generate_tags_list():
    """Generate random list of lb UUIDs"""
    tags = [{'key': 'key-tags', 'value': 'val-tags'}]
    return tags


class TestVLB(test_base.TestCommand):

    def setUp(self):
        super(TestVLB, self).setUp()

        self.app.client_manager.vlb = mock.Mock()
        self.client = self.app.client_manager.vlb


class FakeLoadBalancer(test_base.Fake):
    """Fake one or more loadbalancer"""

    @classmethod
    def generate(cls):
        object_info = {
            "availability_zones": ['eu-de-01'],
            "created_at": uuid.uuid4().hex,
            "description": uuid.uuid4().hex,
            "deletion_protection_enable": True,
            "eips": generate_eips_list(),
            "floating_ip": generate_floating_ips_list()[0],
            "floating_ips": generate_floating_ips_list(),
            "is_guaranteed": True,
            "is_admin_state_up": True,
            "ip_target_enable": True,
            "l4_flavor_id": uuid.uuid4().hex,
            "l7_flavor_id": uuid.uuid4().hex,
            "listeners": generate_listeners_list(),
            "name": uuid.uuid4().hex,
            "network_ids": generate_network_ids_list(),
            "subnet_type": uuid.uuid4().hex,
            "operating_status": uuid.uuid4().hex,
            "pools": generate_pools_list(),
            "project_id": uuid.uuid4().hex,
            "provider": "vlb",
            "provisioning_status": uuid.uuid4().hex,
            "tags": generate_tags_list(),
            "updated_at": uuid.uuid4().hex,
            "ip_address": uuid.uuid4().hex,
            "port_id": uuid.uuid4().hex,
            "subnet_id": uuid.uuid4().hex,
            "vpc_id": uuid.uuid4().hex,
        }

        obj = load_balancer.LoadBalancer.existing(**object_info)
        return obj


def generate_ipgroup():
    """Generate random list of vault UUIDs"""
    ipgroup = {"ipgroup_id": "ipgroup-uuid",
               "enable_ipgroup": True,
               "type": "ipgroup-type"}
    return ipgroup


def generate_loadbalancers_list():
    """Generate random list of loadbalancers UUIDs"""
    loadbalancers = [{"id": 'loadbalancer-id-1'},
                     {"id": 'loadbalancer-id-2'}]
    return loadbalancers


class FakeListener(test_base.Fake):
    """Fake one or more listener"""

    @classmethod
    def generate(cls):
        object_info = {
            "client_ca_tls_container_ref": uuid.uuid4().hex,
            "client_timeout": uuid.uuid4().hex,
            "created_at": uuid.uuid4().hex,
            "connection_limit": 10,
            "default_pool_id": uuid.uuid4().hex,
            "default_tls_container_ref": uuid.uuid4().hex,
            "enable_member_retry": True,
            "enhance_l7policy": True,
            "http2_enable": True,
            "insert_headers": {},
            "is_admin_state_up": True,
            "load_balancers": generate_loadbalancers_list(),
            "ipgroup": generate_ipgroup(),
            "name": uuid.uuid4().hex,
            "keepalive_timeout": 10,
            "member_timeout": 10,
            "protocol": uuid.uuid4().hex,
            "protocol_port": uuid.uuid4().hex,
            "project_id": uuid.uuid4().hex,
            "security_policy_id": uuid.uuid4().hex,
            "sni_container_refs": [],
            "sni_match_algo": uuid.uuid4().hex,
            "tags": generate_tags_list(),
            "transparent_client_ip_enable": True,
            "tls_ciphers_policy": uuid.uuid4().hex,
            "updated_at": uuid.uuid4().hex,
        }

        obj = listener.Listener.existing(**object_info)
        return obj



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from otcextensions.common import sdk_utils
from otcextensions.osclient.load_balancer.v1 import load_balancer
from otcextensions.tests.unit.osclient.load_balancer.v1 import fakes


class TestListLoadBalancer(fakes.TestLoadBalancer):

    _objects = fakes.FakeLoadBalancer.create_multiple(3)

    columns = ('id', 'name', 'project_id', 'vip_address',
               'provisioning_status', 'provider')

    data = []

    for s in _objects:
        data.append((
            s.id,
            s.name,
            s.project_id,
            s.vip_address,
            s.provisioning_status,
            s.provider,
        ))

    def setUp(self):
        super(TestListLoadBalancer, self).setUp()

        self.cmd = load_balancer.ListLoadBalancer(self.app, None)

        self.client.load_balancers = mock.Mock()

    def test_list_default(self):
        arglist = [
        ]

        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.load_balancers.side_effect = [
            self._objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.load_balancers.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowLoadBalancer(fakes.TestLoadBalancer):

    _object = fakes.FakeLoadBalancer.create_one()

    columns = ('admin_state_up', 'description', 'id',
               'listener_ids', 'name', 'operating_status', 'pool_ids',
               'project_id', 'provider', 'provisioning_status', 'vip_address',
               'vip_port_id', 'vip_subnet_id')

    data = (
        _object.is_admin_state_up,
        _object.description,
        _object.id,
        sdk_utils.ListOfIdsColumnBR(_object.listener_ids),
        _object.name,
        _object.operating_status,
        sdk_utils.ListOfIdsColumnBR(_object.pool_ids),
        _object.project_id,
        _object.provider,
        _object.provisioning_status,
        _object.vip_address,
        _object.vip_port_id,
        _object.vip_subnet_id,
    )

    def setUp(self):
        super(TestShowLoadBalancer, self).setUp()

        self.cmd = load_balancer.ShowLoadBalancer(self.app, None)

        self.client.find_load_balancer = mock.Mock()

    def test_show_default(self):
        arglist = [
            'lb'
        ]

        verifylist = [
            ('load_balancer', 'lb')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_load_balancer.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_load_balancer.assert_called_once_with(
            name_or_id='lb',
            ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestCreateLoadBalancer(fakes.TestLoadBalancer):

    _object = fakes.FakeLoadBalancer.create_one()

    columns = (
        'admin_state_up',
        'description',
        'id',
        'listener_ids',
        'name',
        'operating_status',
        'pool_ids',
        'project_id',
        'provider',
        'provisioning_status',
        'vip_address',
        'vip_port_id',
        'vip_subnet_id')

    data = (
        _object.is_admin_state_up,
        _object.description,
        _object.id,
        sdk_utils.ListOfIdsColumnBR(_object.listener_ids),
        _object.name,
        _object.operating_status,
        sdk_utils.ListOfIdsColumnBR(_object.pool_ids),
        _object.project_id,
        _object.provider,
        _object.provisioning_status,
        _object.vip_address,
        _object.vip_port_id,
        _object.vip_subnet_id,
    )

    def setUp(self):
        super(TestCreateLoadBalancer, self).setUp()

        self.cmd = load_balancer.CreateLoadBalancer(self.app, None)

        self.client.create_load_balancer = mock.Mock()

    def test_create_default(self):
        arglist = [
            '--disable',
            '--description', 'descr',
            '--name', 'nm',
            '--vip_address', 'vip_addr',
            '--vip_network_id', 'vip_network_id',
            '--vip_qos_policy_id', 'vip_qos_policy_id'
        ]

        verifylist = [
            ('disable', True),
            ('description', 'descr'),
            ('name', 'nm'),
            ('vip_address', 'vip_addr'),
            ('vip_network_id', 'vip_network_id'),
            ('vip_qos_policy_id', 'vip_qos_policy_id'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_load_balancer.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_load_balancer.assert_called_once_with(
            is_admin_state_up=False,
            description='descr',
            name='nm',
            vip_address='vip_addr',
            vip_network_id='vip_network_id',
            vip_qos_policy_id='vip_qos_policy_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_create_subnet(self):
        arglist = [
            '--vip_subnet_id', 'vip_subnet_id',
        ]

        verifylist = [
            ('vip_subnet_id', 'vip_subnet_id'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_load_balancer.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_load_balancer.assert_called_once_with(
            vip_subnet_id='vip_subnet_id',
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_create_port(self):
        arglist = [
            '--vip_port_id', 'vip_port_id',
        ]

        verifylist = [
            ('vip_port_id', 'vip_port_id'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_load_balancer.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_load_balancer.assert_called_once_with(
            vip_port_id='vip_port_id',
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestUpdateLoadBalancer(fakes.TestLoadBalancer):

    _object = fakes.FakeLoadBalancer.create_one()

    columns = (
        'admin_state_up', 'description', 'id',
        'listener_ids', 'name', 'operating_status',
        'pool_ids', 'project_id', 'provider',
        'provisioning_status', 'vip_address',
        'vip_port_id', 'vip_subnet_id')

    data = (
        _object.is_admin_state_up,
        _object.description,
        _object.id,
        sdk_utils.ListOfIdsColumnBR(_object.listener_ids),
        _object.name,
        _object.operating_status,
        sdk_utils.ListOfIdsColumnBR(_object.pool_ids),
        _object.project_id,
        _object.provider,
        _object.provisioning_status,
        _object.vip_address,
        _object.vip_port_id,
        _object.vip_subnet_id,
    )

    def setUp(self):
        super(TestUpdateLoadBalancer, self).setUp()

        self.cmd = load_balancer.SetLoadBalancer(self.app, None)

        self.client.update_load_balancer = mock.Mock()

    def test_update_default(self):
        arglist = [
            'lb',
            '--disable',
            '--description', 'descr',
            '--name', 'nm',
            '--vip_qos_policy_id', 'vip_qos_policy_id'
        ]

        verifylist = [
            ('load_balancer', 'lb'),
            ('disable', True),
            ('description', 'descr'),
            ('name', 'nm'),
            ('vip_qos_policy_id', 'vip_qos_policy_id'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_load_balancer.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_load_balancer.assert_called_once_with(
            load_balancer='lb',
            is_admin_state_up=False,
            description='descr',
            name='nm',
            vip_qos_policy_id='vip_qos_policy_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestDeleteLoadBalancer(fakes.TestLoadBalancer):

    _object = fakes.FakeLoadBalancer.create_one()

    def setUp(self):
        super(TestDeleteLoadBalancer, self).setUp()

        self.cmd = load_balancer.DeleteLoadBalancer(self.app, None)

        self.client.delete_load_balancer = mock.Mock()
        self.client.find_load_balancer = mock.Mock()

    def test_create_default(self):
        arglist = [
            'lb',
        ]

        verifylist = [
            ('load_balancer', ['lb']),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_load_balancer.side_effect = [
            {}
        ]
        self.client.find_load_balancer.side_effect = [
            self._object
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_load_balancer.assert_called_once_with(
            load_balancer=self._object.id,
            ignore_missing=False
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import exceptions
from openstack import resource


class TagsSpec(resource.Resource):
    #: Tag key.
    key = resource.Body('key')
    #: Tag value.
    value = resource.Body('value')


class PortSpec(resource.Resource):
    #: Port for accessing the VPC endpoint.
    client_port = resource.Body('client_port', type=int)
    #: Port mapping protocol.
    protocol = resource.Body('protocol')
    #: Port for accessing the VPC endpoint service.
    server_port = resource.Body('server_port', type=int)


class Service(resource.Resource):
    base_path = '/vpc-endpoint-services'
    resources_key = 'endpoint_services'

    _query_mapping = resource.QueryParameters(
        'id',
        'limit',
        'name',
        'offset',
        'sort_key',
        'sort_dir',
        'status',
        name='endpoint_service_name',
    )

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    # Properties
    #: Creation time of the VPC endpoint service.
    created_at = resource.Body('created_at')
    #: Count of Creating or Accepted VPC endpoints under the
    #:  VPC endpoint service.
    connection_count = resource.Body('connection_count', type=int)
    #: Supplementary information about the VPC endpoint service.
    description = resource.Body('description')
    #: Specifies the user's domain ID.
    domain_id = resource.Body('domain_id')
    #: ID of the VPC endpoint service.
    id = resource.Body('id')
    #: Whether connection approval is required.
    is_approval_enabled = resource.Body('approval_enabled', type=bool)
    #: ID of the cluster associated with the target VPCEP resource.
    pool_id = resource.Body('pool_id')
    #: ID for identifying the backend resource of the VPC endpoint service.
    port_id = resource.Body('port_id')
    #: Lists the port mappings opened to the VPC endpoint service.
    ports = resource.Body('ports', type=list, list_type=PortSpec)
    #: Project ID.
    project_id = resource.Body('project_id')
    #: ID of the router to which the backend resource of the VPC endpoint
    #:  service belongs.
    router_id = resource.Body('vpc_id')
    #: Resource type.
    server_type = resource.Body('server_type')
    #: Name of the VPC endpoint service.
    service_name = resource.Body('service_name')
    #: Type of the VPC endpoint service.
    service_type = resource.Body('service_type')
    #: Status of the VPC endpoint service.
    status = resource.Body('status')
    #: Lists the resource tags.
    tags = resource.Body('tags', type=list, list_type=TagsSpec)
    #: Specifies whether the client IP address and port number or marker_id
    #:  information is transmitted to the server.
    tcp_proxy = resource.Body('tcp_proxy')
    #: Update time of the VPC endpoint service.
    updated_at = resource.Body('updated_at')

    @classmethod
    def _get_one_match(cls, name_or_id, results):
        """Given a list of results, return the match"""
        the_result = None
        for maybe_result in results:
            id_value = cls._get_id(maybe_result)
            name_value = maybe_result.service_name

            if (id_value == name_or_id) or (name_value == name_or_id):
                # Only allow one resource to be found. If we already
                # found a match, raise an exception to show it.
                if the_result is None:
                    the_result = maybe_result
                else:
                    msg = "More than one %s exists with the name '%s'."
                    msg = msg % (cls.__name__, name_or_id)
                    raise exceptions.DuplicateResource(msg)
        return the_result



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Certificate(resource.Resource):
    resource_key = 'certificate'
    resources_key = 'certificates'
    base_path = '/elb/certificates'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id', 'name', 'description', 'admin_state_up',
        'domain', 'type', is_admin_state_up='admin_state_up'
    )

    # Properties
    #: Specifies the private key of the certificate.
    certificate = resource.Body('certificate')
    #: Specifies the time when the certificate was created.
    created_at = resource.Body('created_at')
    #: Provides supplementary information about the certificate.
    description = resource.Body('description')
    #: Specifies the domain names used by the server certificate.
    domain = resource.Body('domain')
    #: Specifies the enterprise project ID.
    enterprise_project_id = resource.Body('enterprise_project_id')
    #: Specifies the time when the certificate expires.
    expire_time = resource.Body('expire_time')
    #: Specifies the administrative status of the certificate.
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies the private key of the server certificate.
    private_key = resource.Body('private_key')
    #: Specifies the ID of the project where the certificate is used.
    project_id = resource.Body('project_id')
    #: Specifies the certificate type.
    type = resource.Body('type')
    #: Specifies the time when the certificate was updated.
    updated_at = resource.Body('updated_at')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk import sdk_resource


class Job(sdk_resource.Resource):
    """Volume backup Job Resource"""
    base_path = "/jobs"

    # capabilities
    allow_get = True

    #: Properties
    id = resource.Body("job_id", alternate_id=True)
    type = resource.Body("job_type")
    begin_time = resource.Body("begin_time")
    end_time = resource.Body("end_time")
    entities = resource.Body("entities")
    status = resource.Body("status")
    error_code = resource.Body("error_code")
    fail_reason = resource.Body("fail_reason")
    message = resource.Body("message")
    code = resource.Body("code")
    sub_jobs = resource.Body("sub_jobs")



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
'''
Create CBR vault with attributes
'''
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


attrs = {
    'description': 'my vault',
    'auto_bind': False,
    'bind_rules': {},
    'name': 'vault-name',
    'billing': {
        'protect_type': 'backup',
        'object_type': 'server',
        'size': 40,
        'cloud_type': 'public',
        'consistent_level': 'crash_consistent',
        'charging_mode': 'post_paid'
    },
    'auto_expand': False,
    'resources': [
        {
            'name': 'server-name',
            'type': 'OS::Nova::Server',
            'id': 'server-id',
        }
    ]
}

vault = conn.cbr.create_vault(**attrs)
print(vault)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack.tests.unit import test_proxy_base
from otcextensions.sdk.vpcep.v1 import _proxy
from otcextensions.sdk.vpcep.v1 import connection
from otcextensions.sdk.vpcep.v1 import endpoint
from otcextensions.sdk.vpcep.v1 import quota
from otcextensions.sdk.vpcep.v1 import service
from otcextensions.sdk.vpcep.v1 import whitelist
from otcextensions.sdk.vpcep.v1 import public_service
from otcextensions.sdk.vpcep.v1 import target_service


class TestVpcepProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestVpcepProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestEndpointService(TestVpcepProxy):
    def test_list_services(self):
        self.verify_list(
            self.proxy.services,
            service.Service,
        )

    def test_create_service(self):
        self.verify_create(
            self.proxy.create_service,
            service.Service,
            method_kwargs={'name': 'id'},
            expected_kwargs={'name': 'id'},
        )

    def test_get_service(self):
        self.verify_get(
            self.proxy.get_service,
            service.Service,
        )

    def test_find_service(self):
        self.verify_find(
            self.proxy.find_service,
            service.Service,
        )

    def test_update_service(self):
        self.verify_update(
            self.proxy.update_service,
            service.Service,
        )

    def test_delete_service(self):
        self.verify_delete(
            self.proxy.delete_service,
            service.Service,
            True,
        )


class TestEndpoint(TestVpcepProxy):
    def test_endpoints(self):
        self.verify_list(self.proxy.endpoints, endpoint.Endpoint)

    def test_endpoint_create(self):
        self.verify_create(
            self.proxy.create_endpoint,
            endpoint.Endpoint,
            method_kwargs={'name': 'id'},
            expected_kwargs={'name': 'id'},
        )

    def test_endpoint_get(self):
        self.verify_get(self.proxy.get_endpoint, endpoint.Endpoint)

    def test_endpoint_delete(self):
        self.verify_delete(self.proxy.delete_endpoint, endpoint.Endpoint, True)


class TestWhitelist(TestVpcepProxy):
    def test_service_whitelist(self):
        self.verify_list(
            self.proxy.service_whitelist,
            whitelist.Whitelist,
            method_kwargs={'service': 'endpoint-service-id'},
            expected_kwargs={'endpoint_service_id': 'endpoint-service-id'},
        )

    def test_manage_service_whitelist_add(self):
        self._verify(
            'otcextensions.sdk.vpcep.v1.whitelist.Whitelist.add',
            self.proxy.manage_service_whitelist,
            method_kwargs={
                'service': 'endpoint-service-id',
                'action': 'add',
                'domains': ['abc', 'xyz'],
            },
            expected_args=[self.proxy, ['abc', 'xyz']],
        )

    def test_manage_service_whitelist_remove(self):
        self._verify(
            'otcextensions.sdk.vpcep.v1.whitelist.Whitelist.remove',
            self.proxy.manage_service_whitelist,
            method_kwargs={
                'service': 'endpoint-service-id',
                'action': 'remove',
                'domains': ['abc', 'xyz'],
            },
            expected_args=[self.proxy, ['abc', 'xyz']],
        )


class TestConnection(TestVpcepProxy):
    def test_service_connections(self):
        self.verify_list(
            self.proxy.service_connections,
            connection.Connection,
            method_kwargs={'service': 'endpoint-service-id'},
            expected_kwargs={'endpoint_service_id': 'endpoint-service-id'},
        )

    def test_manage_service_connections_accept(self):
        self._verify(
            'otcextensions.sdk.vpcep.v1.connection.Connection.accept',
            self.proxy.manage_service_connections,
            method_kwargs={
                'service': 'endpoint-service-id',
                'action': 'accept',
                'endpoints': ['abc', 'xyz'],
            },
            expected_args=[self.proxy, ['abc', 'xyz']],
        )

    def test_manage_service_connections_reject(self):
        self._verify(
            'otcextensions.sdk.vpcep.v1.connection.Connection.reject',
            self.proxy.manage_service_connections,
            method_kwargs={
                'service': 'endpoint-service-id',
                'action': 'reject',
                'endpoints': ['abc', 'xyz'],
            },
            expected_args=[self.proxy, ['abc', 'xyz']],
        )


class TestQuota(TestVpcepProxy):
    def test_resource_quota(self):
        self.verify_list(self.proxy.resource_quota, quota.Quota)


class TestPublicService(TestVpcepProxy):
    def test_public_services(self):
        self.verify_list(
            self.proxy.public_services, public_service.PublicService
        )


class TestTargetService(TestVpcepProxy):
    def test_get_target_service(self):
        resource_id = 'resource_id'
        base_path_id = target_service.TargetService.base_path + \
            '?id={}'.format(resource_id)

        self._verify(
            'otcextensions.sdk.vpcep.v1._proxy.Proxy._get',
            self.proxy.get_target_service,
            method_args=[resource_id],
            expected_args=[target_service.TargetService],
            expected_kwargs={
                'base_path': base_path_id,
                'requires_id': False,
            }
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class BackendServer(resource.Resource):
    base_path = (f'apigw/instances/%(gateway_id)s/vpc-channels/'
                 f'%(vpc_chan_id)s/members')
    resources_key = "members"

    allow_delete = True
    allow_list = True
    allow_commit = True

    gateway_id = resource.URI('gateway_id')
    vpc_chan_id = resource.URI('vpc_chan_id')

    host = resource.Body('host')
    weight = resource.Body('weight', type=int)
    is_backup = resource.Body('is_backup', type=bool)
    member_group_name = resource.Body('member_group_name')
    status = resource.Body('status', type=int)
    port = resource.Body('port', type=int)
    ecs_id = resource.Body('ecs_id')
    ecs_name = resource.Body('ecs_name')
    id = resource.Body('id')
    vpc_channel_id = resource.Body('vpc_channel_id')
    create_time = resource.Body('create_time')
    member_group_id = resource.Body('member_group_id')

    def create_members(self, session, gateway_id, vpc_channel_id, **attrs):
        return self.send_request(
            session,
            gateway_id,
            vpc_channel_id,
            'POST',
            **attrs
        )

    def update_members(self, session, gateway_id, vpc_channel_id, **attrs):
        return self.send_request(
            session,
            gateway_id,
            vpc_channel_id,
            'PUT',
            **attrs
        )

    @staticmethod
    def send_request(session, gateway_id, vpc_channel_id, method,
                     **attrs):
        uri = (
            f"/apigw/instances/{gateway_id}/vpc-channels/"
            f"{vpc_channel_id}/members"
        )
        session_method = session.put if method == 'PUT' else session.post
        response = session_method(uri, json=attrs)
        exceptions.raise_from_response(response)
        result = response.json()['members']
        if isinstance(result, list):
            return [BackendServer.existing(**item) for item in result]
        elif isinstance(result, dict):
            return BackendServer.existing(**result)

    def enable_server(self, session, gateway_id, vpc_channel_id, **attrs):
        uri = f'apigw/instances/{gateway_id}/vpc-channels/'\
              f'{vpc_channel_id}/members/batch-enable'
        response = session.post(uri, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return response

    def disable_server(self, session, gateway_id, vpc_channel_id, **attrs):
        uri = f'apigw/instances/{gateway_id}/vpc-channels/'\
              f'{vpc_channel_id}/members/batch-disable'
        response = session.post(uri, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return response



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.waf.v1 import domain


FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
EXAMPLE = {
    "id": FAKE_ID,
    "hostname": "www.b.com",
    "cname": "3249d21e5eb34d21be12fdc817fcb67d.waf.cloud.com",
    "txt_code": "3249d21e5eb34d21be12fdc817fcb67d",
    "sub_domain": "3249d21e5eb34d21be12fdc817fcb67d.www.b.com",
    "policy_id": "xxxxxxxxxxxxxx",
    "certificate_id": "xxxxxxxxxxxxxxxxxxx",
    "protect_status": 0,
    "access_status": 0,
    "protocol": "HTTP,HTTPS",
    "server": [
        {
            "client_protocol": "HTTPS",
            "server_protocol": "HTTP",
            "address": "X.X.X.X",
            "port": 443
        }, {
            "client_protocol": "HTTP",
            "server_protocol": "HTTP",
            "address": "X.X.X.X",
            "port": 80
        }
    ],
    "proxy": True,
    "sip_header_name": "default",
    "sip_header_list": ["X-Forwarded-For"],
    "timestamp": 1499817600
}


class TestDomain(base.TestCase):

    def setUp(self):
        super(TestDomain, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock()

    def test_basic(self):
        sot = domain.Domain()

        self.assertEqual('/waf/instance', sot.base_path)
        self.assertEqual('items', sot.resources_key)
        self.assertIsNone(sot.resource_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):

        sot = domain.Domain(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['hostname'], sot.name)
        self.assertEqual(EXAMPLE['hostname'], sot.hostname)
        self.assertEqual(EXAMPLE['access_status'], sot.access_status)
        self.assertEqual(EXAMPLE['certificate_id'], sot.certificate_id)
        self.assertEqual(len(EXAMPLE['server']), len(sot.server))

        self.assertDictEqual({
            'hostname': 'hostname',
            'limit': 'limit',
            'marker': 'marker',
            'name': 'hostname',
            'offset': 'offset',
            'policy_name': 'policyname'},
            sot._query_mapping._mapping
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestEnvironment(TestApiG):
    environment = None

    def setUp(self):
        super(TestEnvironment, self).setUp()
        self.create_gateway()

    def test_01_create_environment(self):
        attrs = {
            "name": "DEV",
            "remark": "Development environment"
        }
        environment = self.client.create_environment(
            gateway=TestEnvironment.gateway,
            **attrs)
        TestEnvironment.environment = environment

    def test_02_update_environment(self):
        new_remark = "Updated remark"
        attrs = {
            "name": "DEV",
            "remark": new_remark
        }
        environment = self.client.update_environment(
            environment=TestEnvironment.environment,
            gateway=TestEnvironment.gateway,
            **attrs
        )
        self.assertEqual(environment.remark, new_remark)

    def test_03_list_environments(self):
        environments = list(self.client.environments(
            gateway=TestEnvironment.gateway))
        self.assertGreater(len(environments), 1)

    def test_04_delete_environment(self):
        self.client.delete_environment(gateway=TestEnvironment.gateway,
                                       environment=TestEnvironment.environment,
                                       )
        self.assertEqual(1, len(list(self.client.environments(
            gateway=TestEnvironment.gateway)
        )))
        self.delete_gateway()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import proxy

from otcextensions.common.utils import extract_url_parts
from otcextensions.sdk.vpc.v1 import bandwidth as _bandwidth
from otcextensions.sdk.vpc.v1 import peering as _peering
from otcextensions.sdk.vpc.v1 import route as _route
from otcextensions.sdk.vpc.v1 import subnet as _subnet
from otcextensions.sdk.vpc.v1 import vpc as _vpc

from dataclasses import dataclass
from typing import List


@dataclass
class PublicInfo:
    publicip_id: str
    publicip_type: str


class Proxy(proxy.Proxy):
    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return extract_url_parts(url, project_id)

    # ======== Bandwidth ========
    def assign_bandwidth(self, **attrs):
        """Assign bandwidth

        :param dict attrs: Keyword arguments which will be used to assign
            a :class:`~otcextensions.sdk.vpc.v1.bandwidth.Bandwidth`
        """
        project_id = self.get_project_id()
        version = 'v2.0'
        base_path = _bandwidth.Bandwidth.base_path % {'version': version,
                                                      'project_id': project_id}
        return self._create(_bandwidth.Bandwidth, project_id=project_id,
                            base_path=base_path, **attrs)

    def find_bandwidth(self, name_or_id, ignore_missing=False):
        """Find a single bandwidth

        :param name_or_id: The name or ID of a bandwidth
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the vpc peering does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent peering.

        :returns: One :class:`~otcextensions.sdk.vpc.v1.bandwidth.Bandwidth`
        """
        project_id = self.get_project_id()
        version = 'v1'
        base_path = _bandwidth.Bandwidth.base_path % {'version': version,
                                                      'project_id': project_id}
        return self._find(
            _bandwidth.Bandwidth, name_or_id,
            project_id=project_id, base_path=base_path,
            ignore_missing=ignore_missing)

    def update_bandwidth(self, bandwidth, **attrs):
        """Update a vpc peering

        :param bandwidth: Either the ID of a vpc bandwidth or a
                       :class:`~otcextensions.sdk.vpc.v1.bandwidth.Bandwidth`
                       instance.
        :param dict attrs: The attributes to update on the vpc bandwidth
            represented by ``bandwidth``.

        :returns: The updated bandwidth

        :rtype: :class:`~otcextensions.sdk.vpc.v1.bandwidth.Bandwidth`
        """
        bandwidth = self._get_resource(_bandwidth.Bandwidth, bandwidth)
        project_id = self.get_project_id()
        version = 'v1'
        return bandwidth.update_bandwidth(
            self,
            project_id,
            version,
            **attrs)

    def add_eip_to_bandwidth(self, bandwidth, publicip_info: List[PublicInfo]):
        """Add an EIP to a shared bandwidth.

        :param bandwidth: The value can be the ID of a bandwidth
           or a :class:`~otcextensions.sdk.vpc.v1.bandwidth.Bandwidth`
           instance.
        :param publicip_info: List of dictionaries in the format
           {'publicip_id': id, 'publicip_type': type}
        """
        bandwidth = self._get_resource(_bandwidth.Bandwidth, bandwidth)
        version = 'v2.0'
        return bandwidth.add_eip_to_bandwidth(
            self,
            version,
            publicip_info,
            project_id=self.get_project_id())

    def remove_eip_from_bandwidth(self, bandwidth, **attrs):
        """Add an EIP to a shared bandwidth.

        :param bandwidth: The value can be the ID of a bandwidth
             or a :class:`~otcextensions.sdk.vpc.v1.bandwidth.Bandwidth`
             instance.
        :param attrs: Keyword arguments to remove eip: charge_mode, size,
            publicip_info - array of eip in the format {'publicip_id': id}
        """
        bandwidth = self._get_resource(_bandwidth.Bandwidth, bandwidth)
        project_id = self.get_project_id()
        version = 'v2.0'
        return bandwidth.remove_eip_from_bandwidth(
            self,
            project_id,
            version,
            **attrs)

    def delete_bandwidth(self, bandwidth, ignore_missing=True):
        """Delete a bandwidth

        :param bandwidth: key id or an instance of
            :class:`~otcextensions.sdk.vpc.v1.bandwidth.Bandwidth`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the vpc peering does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent peering.

        :returns: ``None``
        """
        project_id = self.get_project_id()
        version = 'v2.0'
        base_path = _bandwidth.Bandwidth.base_path % {'version': version,
                                                      'project_id': project_id}
        return self._delete(
            _bandwidth.Bandwidth, bandwidth,
            ignore_missing=ignore_missing, project_id=project_id,
            base_path=base_path)

    # ======== Peering ========
    def create_peering(self, **attrs):
        """Create a new vpc peering from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vpc.v1.peering.Peering`
        """
        return self._create(_peering.Peering, **attrs)

    def delete_peering(self, peering, ignore_missing=True):
        """Delete a vpc peering

        :param peering: key id or an instance of
            :class:`~otcextensions.sdk.vpc.v1.peering.Peering`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the vpc peering does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent peering.

        :returns: ``None``
        """
        return self._delete(
            _peering.Peering, peering,
            ignore_missing=ignore_missing)

    def peerings(self, **query):
        """Return a generator of vpc peerings

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of vpc peering objects

        :rtype: :class:`~otcextensions.sdk.vpc.v1.peering.Peering`
        """
        return self._list(_peering.Peering, **query)

    def get_peering(self, peering):
        """Get a single vpc peering

        :param peering: The value can be the ID of a vpc peering or a
                        :class:`~otcextensions.sdk.vpc.v1.peering.Peering`
                        instance.

        :returns: One :class:`~otcextensions.sdk.vpc.v1.peering.Peering`

        :raises: :class:`~openstack.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_peering.Peering, peering)

    def find_peering(self, name_or_id, ignore_missing=False):
        """Find a single vpc peering

        :param name_or_id: The name or ID of a zone
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the vpc peering does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent peering.

        :returns: One :class:`~otcextensions.sdk.vpc.v1.peering.Peering`
        """
        return self._find(
            _peering.Peering, name_or_id,
            ignore_missing=ignore_missing)

    def update_peering(self, peering, **attrs):
        """Update a vpc peering

        :param peering: Either the ID of a vpc peering or a
                       :class:`~otcextensions.sdk.vpc.v1.peering.Peering`
                       instance.
        :param dict attrs: The attributes to update on the vpc peering
            represented by ``peering``.

        :returns: The updated peering

        :rtype: :class:`~otcextensions.sdk.vpc.v1.peering.Peering`
        """
        return self._update(_peering.Peering, peering, **attrs)

    def set_peering(self, peering, set_status):
        """Accept/Reject a vpc peering connection request

        :param peering: Either the ID of a vpc peering or a
                       :class:`~otcextensions.sdk.vpc.v1.peering.Peering`
                       instance.
        :param set_status: The value can been ``accept`` or ``reject``

        :returns: The updated peering

        :rtype: :class:`~otcextensions.sdk.vpc.v1.peering.Peering`
        """
        valid_status = ['accept', 'reject']
        if set_status.lower() not in valid_status:
            raise ValueError(
                "results: status must be one of %r." % valid_status)
        peering = self._get_resource(_peering.Peering, peering)
        return peering._set_peering(self, set_status.lower())

    # ======== Route ========
    def add_route(self, **attrs):
        """Add vpc route

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vpc.v1.route.Route`
        """
        return self._create(_route.Route, **attrs)

    def delete_route(self, route, ignore_missing=True):
        """Delete a vpc route

        :param route: route id or an instance of
            :class:`~otcextensions.sdk.vpc.v1.route.Route`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the vpc route does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent route.

        :returns: ``None``
        """
        return self._delete(_route.Route, route,
                            ignore_missing=ignore_missing)

    def routes(self, **query):
        """Return a generator of vpc routes

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of vpc route objects

        :rtype: :class:`~otcextensions.sdk.vpc.v1.route.Route`
        """
        return self._list(_route.Route, **query)

    def get_route(self, route):
        """Get details of a single vpc route

        :param route: The value can be the ID of a vpc route or a
                        :class:`~otcextensions.sdk.vpc.v1.route.Route`
                        instance.

        :returns: One :class:`~otcextensions.sdk.vpc.v1.route.Route`

        :raises: :class:`~openstack.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(_route.Route, route)

    # ========== VPC ==========
    def vpcs(self, **query):
        """Return a generator of vpcs

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of vpc objects

        :rtype: :class:`~otcextensions.sdk.vpc.v1.vpc.Vpc`
        """
        query['project_id'] = self.get_project_id()
        return self._list(_vpc.Vpc, **query)

    def create_vpc(self, **attrs):
        """Create a new vpc from attributes

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.vpc.v1.vpc.Vpc`
        """
        return self._create(_vpc.Vpc, **attrs,
                            project_id=self.get_project_id())

    def delete_vpc(self, vpc, ignore_missing=True):
        """Delete a vpc

        :param vpc: vpc id or an instance of
            :class:`~otcextensions.sdk.vpc.v1.vpc.Vpc`

        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the vpc route does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent route.

        :returns: none
        """
        return self._delete(_vpc.Vpc, vpc,
                            project_id=self.get_project_id(),
                            ignore_missing=ignore_missing)

    def get_vpc(self, vpc):
        """Get a vpc by id

        :param vpc: vpc id or an instance of
           :class:`~otcextensions.sdk.vpc.v1.vpc.Vpc`

        :returns: One :class:`~otcextensions.sdk.vpc.v1.vpc.Vpc`
        """
        return self._get(_vpc.Vpc, vpc, project_id=self.get_project_id())

    def find_vpc(self, name_or_id, ignore_missing=False):
        """Find a single vpc

        :param name_or_id: The name or ID of a vpc

        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the vpc does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent peering.

        :returns: One :class:`~otcextensions.sdk.vpc.v1.vpc.Vpc`
        """
        return self._find(
            _vpc.Vpc, name_or_id,
            ignore_missing=ignore_missing,
            project_id=self.get_project_id())

    def update_vpc(self, vpc, **attrs):
        """Update vpc

        :param vpc: vpc id or an instance of
            :class:`~otcextensions.sdk.vpc.v1.vpc.Vpc`

        :param dict attrs: The attributes to update on the vpc
            represented by ``vpcd``.
        """
        attrs['project_id'] = self.get_project_id()
        return self._update(_vpc.Vpc, vpc, **attrs)

    # ========== Subnet ==========
    def subnets(self, **query):
        """Return a generator of subnets

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of subnet objects

        :rtype: :class:`~otcextensions.sdk.vpc.v1.subnet.Subnet`
        """
        query['project_id'] = self.get_project_id()
        return self._list(_subnet.Subnet, **query)

    def create_subnet(self, **attrs):
        """Create a new subnet from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.vpc.v1.subnet.Subnet`
        """
        attrs['project_id'] = self.get_project_id()
        return self._create(_subnet.Subnet, **attrs)

    def get_subnet(self, subnet):
        """Get a subnet by id

        :param subnet: subnet id or an instance of
            :class:`~otcextensions.sdk.vpc.v1.subnet.Subnet`

        :returns: One :class:`~otcextensions.sdk.vpc.v1.subnet.Subnet`
        """
        return self._get(_subnet.Subnet, subnet,
                         project_id=self.get_project_id())

    def find_subnet(self, name_or_id, ignore_missing=False):
        """Find a single subnet

        :param name_or_id: The name or ID of a subnet

        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the subnet does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent peering.

        :returns: One :class:`~otcextensions.sdk.vpc.v1.subnet.Subnet`
        """
        return self._find(
            _subnet.Subnet, name_or_id,
            ignore_missing=ignore_missing,
            project_id=self.get_project_id())

    def update_subnet(self, subnet, **attrs):
        """Update subnet

        :param subnet: subnet id or an instance of
            :class:`~otcextensions.sdk.vpc.v1.subnet.Subnet`

        :param dict attrs: The attributes to update on the subnet
            represented by ``subnet``.
        """
        attrs['project_id'] = self.get_project_id()

        rs = self._get_resource(_subnet.Subnet, subnet)
        if rs.vpc_id is None and 'vpc_id' not in attrs:
            raise AttributeError('Updating subnet requires VPC ID')
        vpc_id = attrs.pop('vpc_id', rs.vpc_id)  # vpc_id can't be changed

        attrs.pop('base_path', None)
        base_path = _subnet.vpc_subnet_base_path(vpc_id)

        return self._update(_subnet.Subnet,
                            subnet,
                            base_path=base_path,
                            **attrs)

    def _delete(self, resource_type, value, ignore_missing=True, **attrs):
        """Override of ``_delete`` with support of ``base_path``"""
        base_path = attrs.pop('base_path', None)

        res = self._get_resource(resource_type, value, **attrs)

        try:
            rv = res.delete(self, base_path=base_path)
        except exceptions.ResourceNotFound:
            if ignore_missing:
                return None
            raise

        return rv

    def delete_subnet(self, subnet, vpc_id=None, ignore_missing=True):
        """Delete a subnet

        :param subnet: subnet id or an instance of
            :class:`~otcextensions.sdk.vpc.v1.subnet.Subnet`

        :param vpc_id: VPC id. By default, taken from ``subnet``, if provided.

        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the subnet route does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent route.

        :returns: none
        """
        sn_res = self._get_resource(_subnet.Subnet, subnet)
        sn_res.vpc_id = vpc_id or sn_res.vpc_id
        if sn_res.vpc_id is None:
            raise AttributeError('Deleting subnet requires VPC ID')

        base_path = _subnet.vpc_subnet_base_path(sn_res.vpc_id)

        return self._delete(_subnet.Subnet, subnet,
                            ignore_missing=ignore_missing,
                            base_path=base_path)

    # ========== Project cleanup ==========
    def _get_cleanup_dependencies(self):
        return {
            'vpc': {
                'before': ['network']
            }
        }

    def _service_cleanup(self, dry_run=True, client_status_queue=None,
                         identified_resources=None,
                         filters=None, resource_evaluation_fn=None,
                         skip_resources=None):
        for obj in self.peerings():
            self._service_cleanup_del_res(
                self.delete_peering,
                obj,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import uuid

from openstackclient.tests.functional import base


class TestDirectConnection(base.TestCase):

    UUID = uuid.uuid4().hex[:8]
    DC_NAME = "direct_connect-" + UUID
    BANDWIDTH = 50
    PORT_TYPE = "1G"
    PROVIDER = "OTC"
    LOCATION = "Biere"

    def setUp(self):
        super(TestDirectConnection, self).setUp()

    def tearDown(self):
        try:
            if self.DC_ID:
                self._delete_direct_connection()
        finally:
            super(TestDirectConnection, self).tearDown()

    def _create_direct_connection(self):
        json_output = json.loads(self.openstack(
            'dcaas connection create'
            ' {port_type}'
            ' {bandwidth}'
            ' {location}'
            ' {provider}'
            ' --name {name} -f json'.format(
                name=self.DC_NAME,
                port_type=self.PORT_TYPE,
                bandwidth=self.BANDWIDTH,
                location=self.LOCATION,
                provider=self.PROVIDER
            )
        ))
        self.DC_ID = json_output['id']
        return json_output

    def _delete_direct_connection(self):
        self.openstack('dcaas connection delete ' + self.DC_ID)

    def test_create_direct_connection(self):
        json_output = self._create_direct_connection()
        self.assertIsNotNone(json_output)
        self.assertEqual(json_output['id'], self.DC_ID)

    def test_list_direct_connection(self):
        self._create_direct_connection()
        json_output = json.loads(self.openstack(
            'dcaas connection list -f json'
        ))
        self.assertIsNotNone(json_output)

    def test_list_filter_direct_connection(self):
        self._create_direct_connection()
        json_output = json.loads(self.openstack(
            'dcaas connection list '
            '--name {name} '
            '--bandwidth {bandwidth} '
            '--port_type {port_type} '
            '--location {location} '
            '--provider {provider} -f json'.format(
                name=self.DC_NAME,
                bandwidth=self.BANDWIDTH,
                port_type=self.PORT_TYPE,
                location=self.LOCATION,
                provider=self.PROVIDER
            )
        ))
        self.assertIsNotNone(json_output)

    def test_find_direct_connection(self):
        self._create_direct_connection()
        json_output = json.loads(self.openstack(
            'dcaas connection show {id} -f json'.format(id=self.DC_ID)
        ))
        self.assertIsNotNone(json_output)

    def test_update_direct_connection(self):
        self._create_direct_connection()
        connection_name = self.DC_NAME + '-updated'
        json_output = json.loads(self.openstack(
            'dcaas connection update {id} --name {name} -f json'.format(
                id=self.DC_ID,
                name=connection_name
            )
        ))
        self.assertEqual(json_output['id'], self.DC_ID)
        self.assertEqual(json_output['name'], connection_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import resource_query as _rq

EXAMPLE = {
    "apis_num": 27,
    "apis_in_release": 11,
    "apis_not_in_release": 6
}


class TestApiQuantity(base.TestCase):

    def test_basic(self):
        sot = _rq.ApiQuantities()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/resources/outline/apis',
            sot.base_path)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = _rq.ApiQuantities(**EXAMPLE)
        self.assertEqual(EXAMPLE['apis_num'], sot.apis_num)
        self.assertEqual(EXAMPLE['apis_in_release'], sot.apis_in_release)
        self.assertEqual(
            EXAMPLE['apis_not_in_release'], sot.apis_not_in_release
        )


EXAMPLE_GROUP = {
    "not_in_koogallery": 0,
    "in_koogallery": 23
}


class TestApiGroupQuantity(base.TestCase):

    def test_basic(self):
        sot = _rq.ApiGroupQuantities()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/resources/outline/groups',
            sot.base_path)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = _rq.ApiGroupQuantities(**EXAMPLE_GROUP)
        self.assertEqual(
            EXAMPLE_GROUP['not_in_koogallery'], sot.not_in_koogallery
        )
        self.assertEqual(EXAMPLE_GROUP['in_koogallery'], sot.in_koogallery)


EXAMPLE_APP = {
    "authorized": 7,
    "not_authorized": 5
}


class TestAppQuantity(base.TestCase):

    def test_basic(self):
        sot = _rq.AppQuantities()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/resources/outline/apps',
            sot.base_path)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = _rq.AppQuantities(**EXAMPLE_APP)
        self.assertEqual(EXAMPLE_APP['authorized'], sot.authorized)
        self.assertEqual(EXAMPLE_APP['not_authorized'], sot.not_authorized)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock
from keystoneauth1 import adapter

from openstack import resource
from openstack.tests.unit import base
from openstack.tests.unit import test_resource

from otcextensions.sdk.deh.v1 import host


FAKE_ID = '68d5745e-6af2-40e4-945d-fe449be00148'
EXAMPLE = {
    'dedicated_host_id': FAKE_ID,
    'name': 'win_2008 servers',
    'auto_placement': 'off',
    'availability_zone': 'az1',
    'host_properties': {
        'vcpus': 36,
        'cores': 12,
        'sockets': 2,
        'memory': 1073741824,
        'host_type': 'h1',
        'host_type_name': 'High performance',
        'available_instance_capacities': [
            {
                'flavor': 'h1.large'
            },
            {
                'flavor': 'h1.2large'
            },
            {
                'flavor': 'h1.4large'
            },
            {
                'flavor': 'h1.8large'
            }
        ]
    },
    'state': 'available',
    'project_id': '9c53a566cb3443ab910cf0daebca90c4',
    'available_vcpus': 20,
    'available_memory': 1073201821,
    'instance_total': 2,
    'allocated_at': '2016-10-10T14:35:47Z',
    'released_at': None,
    'instance_uuids': [
        'erf5th66cb3443ab912ff0daebca3456',
        '23457h66cb3443ab912ff0daebcaer45'
    ]
}


class TestHost(base.TestCase):

    def setUp(self):
        super(TestHost, self).setUp()
        self.sot = host.Host(id='id')

        self.request = mock.Mock(spec=resource._Request)
        self.request.url = "uri"
        self.request.body = "body"
        self.request.headers = "headers"

        self.response = test_resource.FakeResponse({})

        self.sot._prepare_request = mock.Mock(return_value=self.request)
        self.sot._translate_response = mock.Mock()

        self.session = mock.Mock(spec=adapter.Adapter)
        self.session.get = mock.Mock(return_value=self.response)
        self.session.post = mock.Mock(return_value=self.response)

    def test_basic(self):
        sot = host.Host()

        self.assertEqual('/dedicated-hosts', sot.base_path)

        self.assertDictEqual({'availability_zone': 'availability_zone',
                              'changes_since': 'changes-since',
                              'flavor': 'flavor',
                              'host_type': 'host_type',
                              'host_type_name': 'host_type_name',
                              'id': 'id',
                              'instance_uuid': 'instance_uuid',
                              'limit': 'limit',
                              'marker': 'marker',
                              'name': 'name',
                              'released_at': 'released_at',
                              'status': 'status',
                              'tags': 'tags',
                              'tenant': 'tenant'},
                             sot._query_mapping._mapping)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):

        sot = host.Host(**EXAMPLE)
        self.assertEqual(EXAMPLE['dedicated_host_id'], FAKE_ID)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['auto_placement'], sot.auto_placement)
        self.assertEqual(EXAMPLE['availability_zone'], sot.availability_zone)
        self.assertEqual(EXAMPLE['state'], sot.status)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['available_vcpus'], sot.available_vcpus)
        self.assertEqual(EXAMPLE['available_memory'], sot.available_memory)
        self.assertEqual(EXAMPLE['instance_total'], sot.instance_total)
        self.assertEqual(EXAMPLE['released_at'], sot.released_at)
        ref = EXAMPLE['host_properties']
        actual = sot.host_properties
        self.assertEqual(ref['vcpus'], actual.vcpus)
        self.assertEqual(ref['cores'], actual.cores)
        self.assertEqual(ref['sockets'], actual.sockets)
        self.assertEqual(ref['memory'], actual.memory)
        self.assertEqual(ref['host_type'], actual.host_type)
        self.assertEqual(ref['host_type_name'], actual.host_type_name)

    def test_fetch_tags(self):
        res = self.sot
        sess = self.session

        tags = [
            {'key': 'color', 'value': 'blue'},
            {'key': 'color', 'value': 'green'}
        ]

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.links = {}
        mock_response.json.return_value = {
            'tags': tags}

        sess.get.side_effect = [mock_response]

        result = res.fetch_tags(sess)
        # Check tags attribute is updated
        self.assertEqual(tags, res.tags)
        # Check the passed resource is returned
        self.assertEqual(res, result)
        url = 'dedicated-host-tags/' + res.id + '/tags'
        sess.get.assert_called_once_with(url)

    def test_add_tags(self):
        res = self.sot
        sess = self.session

        tags = [
            {'key': 'color', 'value': 'blue'},
            {'key': 'color', 'value': 'green'}
        ]

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.links = {}
        mock_response.json.return_value = {
            'tags': tags}

        sess.get.side_effect = [mock_response]

        result = res.add_tags(sess, tags)
        # Check tags attribute is updated
        self.assertEqual(tags, res.tags)
        # Check the passed resource is returned
        self.assertEqual(res, result)
        url = 'dedicated-host-tags/' + res.id + '/tags/action'
        sess.post.assert_called_once_with(
            url,
            json={'action': 'create', 'tags': tags}
        )

    def test_remove_tags(self):
        res = self.sot
        sess = self.session

        tags = [
            {'key': 'color', 'value': 'blue'},
            {'key': 'color', 'value': 'green'}
        ]

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.links = {}
        mock_response.json.return_value = {
            'tags': []}

        sess.get.side_effect = [mock_response]

        result = res.remove_tags(sess, tags)
        # Check tags attribute is updated
        self.assertEqual([], res.tags)
        # Check the passed resource is returned
        self.assertEqual(res, result)
        url = 'dedicated-host-tags/' + res.id + '/tags/action'
        sess.post.assert_called_once_with(
            url,
            json={'action': 'delete', 'tags': tags}
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


from otcextensions.sdk.auto_scaling.v1 import _base


class ScheduledPolicy(resource.Resource):
    #: schedule time,
    #:  ** if policy type is ``SCHEDULED``, launch time should be
    #:          ``YYYY-MM-DDThh:mmZ``,
    #:  ** if policy type is ``RECURRENCE``, launch time should be ``hh:mm``
    launch_time = resource.Body('launch_time')
    #: Recurrence Type,
    #: valid values include: ``Daily``, ``Weekly``, ``Monthly``
    recurrence_type = resource.Body('recurrence_type')
    #: Used in concert with ``recurrence_type``,
    #:  ** if recurrence_type is Daily, recurrence_value has no meaning
    #:  ** if recurrence_type is Weekly, recurrence_value means days of week,
    #:       1,3,5 means scheduled at monday, Wednesday, Friday of every week
    #:  ** if recurrence_type is Monthly, recurrence_value means days of month
    #:       1,10,30 means scheduled at first, 10th, 30th day of every month
    recurrence_value = resource.Body('recurrence_value')
    #: policy effect start time (UTC)
    start_time = resource.Body('start_time')
    #: policy effect end time (UTC)
    end_time = resource.Body('end_time')


class Action(resource.Resource):
    #: Scaling trigger action type
    #: valid values include: ``ADD``, ``REMOVE``, ``SET``
    operation = resource.Body('operation')
    #: Number of instances which will be operated by the action
    #: Values from 0 to 200 are possible.
    #: Note: Use either instance_number or instance_percentage
    #: If nothing of instance_number or instance_percentage is set, the
    #: default value is 1.
    instance_number = resource.Body('instance_number', type=int)
    #: Percentage of instances which are currently there to be operated
    #: by the action
    #: Values from 0 to 20000 are possible.
    #: Note: Use either instance_number or instance_percentage
    #: If nothing of instance_number or instance_percentage is set, the default
    #: value is 1.
    instance_percentage = resource.Body('instance_percentage', type=int)


class Policy(_base.Resource):
    """AutoScaling Policy Resource"""
    resource_key = 'scaling_policy'
    resources_key = 'scaling_policies'
    base_path = '/scaling_policy'
    query_marker_key = 'start_number'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'limit', 'name', 'type', 'marker',
        name='scaling_policy_name',
        type='scaling_policy_type',
        marker=query_marker_key,
    )

    #: Properties
    #: AutoScaling policy ID
    id = resource.Body('scaling_policy_id', alternate_id=True)
    #: AutoScaling policy name
    name = resource.Body('scaling_policy_name')
    #: AutoScaling policy trigger type
    #: valid values include: ``ALARM``, ``SCHEDULED``, ``RECURRENCE``
    type = resource.Body('scaling_policy_type')
    #: AutoScaling group reference the policy apply to
    scaling_group_id = resource.Body('scaling_group_id')

    alarm_id = resource.Body('alarm_id')
    scheduled_policy = resource.Body('scheduled_policy',
                                     type=ScheduledPolicy)
    scaling_policy_action = resource.Body('scaling_policy_action',
                                          type=Action)
    cool_down_time = resource.Body('cool_down_time', type=int)
    create_time = resource.Body('create_time')
    #: valid values include: ``INSERVICE``, ``PAUSED``
    status = resource.Body('policy_status')

    def execute(self, session):
        """execute policy"""
        body = {"action": "execute"}
        self._action(session, body)

    def pause(self, session):
        """pause policy"""
        body = {"action": "pause"}
        self._action(session, body)

    def resume(self, session):
        """resume policy"""
        body = {"action": "resume"}
        self._action(session, body)

    @classmethod
    def find(cls, session, name_or_id, ignore_missing=True, **params):
        """Find a resource by its name or id.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param name_or_id: This resource's identifier, if needed by
                           the request. The default is ``None``.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :param dict params: Any additional parameters to be passed into
                            underlying methods, such as to
                            :meth:`~openstack.resource.Resource.existing`
                            in order to pass on URI parameters.

        :return: The :class:`Resource` object matching the given name or id
                 or None if nothing matches.
        :raises: :class:`openstack.exceptions.DuplicateResource` if more
                 than one resource is found for this request.
        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
                 is found and ignore_missing is ``False``.
        """
        session = cls._get_session(session)
        # Try to short-circuit by looking directly for a matching ID.
        group_id = params.pop('group_id', None)
        try:
            match = cls.existing(
                id=name_or_id,
                connection=session._get_connection(),
                **params)
            return match.fetch(session, **params)
        except exceptions.NotFoundException:
            pass

        # if ('name' in cls._query_mapping._mapping.keys()
        #       and 'name' not in params):
        params['name'] = name_or_id

        base_path = '/scaling_policy/{id}/list'.format(id=group_id)
        data = cls.list(session,
                        base_path=base_path,
                        **params)

        result = cls._get_one_match(name_or_id, data)
        if result is not None:
            return result

        if ignore_missing:
            return None
        raise exceptions.ResourceNotFound(
            "No %s found for %s" % (cls.__name__, name_or_id))
